@@ -59,6 +59,7 @@ EXPECTED_POWER_STATES = {
 """Mapping of target power states to expected power states."""
 
 STATE_VERSIONS = {
+    'available': '1.1',
     'enroll': '1.11',
     'manageable': '1.4',
 }
@@ -79,6 +80,9 @@ CONFIG_DRIVE_DICT_VERSION = '1.56'
 DEPLOY_STEPS_VERSION = '1.69'
 """API version in which deploy_steps was added to node provisioning."""
 
+CHANGE_BOOT_MODE_VERSION = '1.76'
+"""API version in which boot_mode and secure_boot states can be changed"""
+
 
 class ListMixin:
 
