@@ -257,12 +257,21 @@ class Node(_common.ListMixin, resource.Resource):
 
         The overridden version is capable of handling the populated
         ``provision_state`` field of one of three values: ``enroll``,
-        ``manageable`` or ``available``. The default is currently
-        ``available``, since it's the only state supported by all API versions.
+        ``manageable`` or ``available``. If not provided, the server default
+        is used (``enroll`` in newer versions of Ironic).
+
+        This call does not cause a node to go through automated cleaning.
+        If you need it, use ``provision_state=manageable`` followed by
+        a call to :meth:`set_provision_state`.
 
         Note that Bare Metal API 1.4 is required for ``manageable`` and
         1.11 is required for ``enroll``.
 
+        .. warning::
+            Using ``provision_state=available`` is only possible with API
+            versions 1.1 to 1.10 and thus is incompatible with setting any
+            fields that appeared after 1.11.
+
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
 
@@ -274,44 +283,41 @@ class Node(_common.ListMixin, resource.Resource):
             supported by the server.
         """
         expected_provision_state = self.provision_state
-        if expected_provision_state is None:
-            expected_provision_state = 'available'
-
-        if expected_provision_state not in ('enroll',
-                                            'manageable',
-                                            'available'):
-            raise ValueError(
-                "Node's provision_state must be one of 'enroll', "
-                "'manageable' or 'available' for creation, got %s" %
-                expected_provision_state)
 
         session = self._get_session(session)
-        # Verify that the requested provision state is reachable with the API
-        # version we are going to use.
-        try:
-            expected_version = _common.STATE_VERSIONS[expected_provision_state]
-        except KeyError:
-            pass
+        if expected_provision_state is not None:
+            # Verify that the requested provision state is reachable with
+            # the API version we are going to use.
+            try:
+                microversion = _common.STATE_VERSIONS[
+                    expected_provision_state]
+            except KeyError:
+                raise ValueError(
+                    "Node's provision_state must be one of %s for creation, "
+                    "got %s" % (', '.join(_common.STATE_VERSIONS),
+                                expected_provision_state))
+            else:
+                error_message = ("Cannot create a node with initial provision "
+                                 "state %s" % expected_provision_state)
+                # Nodes cannot be created as available using new API versions
+                maximum = ('1.10' if expected_provision_state == 'available'
+                           else None)
+                microversion = self._assert_microversion_for(
+                    session, 'create', microversion, maximum=maximum,
+                    error_message=error_message,
+                )
         else:
-            self._assert_microversion_for(
-                session, 'create', expected_version,
-                error_message="Cannot create a node with initial provision "
-                "state %s" % expected_provision_state)
+            microversion = None  # use the base negotiation
 
         # Ironic cannot set provision_state itself, so marking it as unchanged
         self._clean_body_attrs({'provision_state'})
-        super(Node, self).create(session, *args, **kwargs)
 
-        if (self.provision_state == 'enroll'
-                and expected_provision_state != 'enroll'):
-            self.set_provision_state(session, 'manage', wait=True)
+        super(Node, self).create(session, *args, microversion=microversion,
+                                 **kwargs)
 
-        if (self.provision_state == 'manageable'
-                and expected_provision_state == 'available'):
-            self.set_provision_state(session, 'provide', wait=True)
-
-        if (self.provision_state == 'available'
-                and expected_provision_state == 'manageable'):
+        if (expected_provision_state == 'manageable'
+                and self.provision_state != 'manageable'):
+            # Manageable is not reachable directly
             self.set_provision_state(session, 'manage', wait=True)
 
         return self
@@ -768,7 +774,7 @@ class Node(_common.ListMixin, resource.Resource):
             fails for a required interface.
         """
         session = self._get_session(session)
-        version = self._get_microversion_for(session, 'fetch')
+        version = self._get_microversion(session, action='fetch')
 
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'validate')
@@ -818,7 +824,7 @@ class Node(_common.ListMixin, resource.Resource):
 
     def _do_maintenance_action(self, session, verb, body=None):
         session = self._get_session(session)
-        version = self._get_microversion_for(session, 'commit')
+        version = self._get_microversion(session, action='commit')
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'maintenance')
         response = getattr(session, verb)(
@@ -835,10 +841,9 @@ class Node(_common.ListMixin, resource.Resource):
         :param boot_device: Boot device to assign to the node.
         :param persistent: If the boot device change is maintained after node
             reboot
-        :return: The updated :class:`~openstack.baremetal.v1.node.Node`
         """
         session = self._get_session(session)
-        version = self._get_microversion_for(session, 'commit')
+        version = self._get_microversion(session, action='commit')
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'management', 'boot_device')
 
@@ -919,7 +924,6 @@ class Node(_common.ListMixin, resource.Resource):
 
         :param session: The session to use for making this request.
         :param trait: The trait to add to the node.
-        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
         """
         session = self._get_session(session)
         version = utils.pick_microversion(session, '1.37')
@@ -945,7 +949,8 @@ class Node(_common.ListMixin, resource.Resource):
             :class:`~openstack.exceptions.ResourceNotFound` will be
             raised when the trait does not exist.
             Otherwise, ``False`` is returned.
-        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        :returns bool: True on success removing the trait.
+            False when the trait does not exist already.
         """
         session = self._get_session(session)
         version = utils.pick_microversion(session, '1.37')
@@ -956,7 +961,7 @@ class Node(_common.ListMixin, resource.Resource):
             request.url, headers=request.headers, microversion=version,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
 
-        if ignore_missing or response.status_code == 400:
+        if ignore_missing and response.status_code == 400:
             session.log.debug(
                 'Trait %(trait)s was already removed from node %(node)s',
                 {'trait': trait, 'node': self.id})
@@ -966,7 +971,8 @@ class Node(_common.ListMixin, resource.Resource):
                .format(node=self.id, trait=trait))
         exceptions.raise_from_response(response, error_message=msg)
 
-        self.traits = list(set(self.traits) - {trait})
+        if self.traits:
+            self.traits = list(set(self.traits) - {trait})
 
         return True
 
@@ -978,7 +984,6 @@ class Node(_common.ListMixin, resource.Resource):
 
         :param session: The session to use for making this request.
         :param traits: list of traits to add to the node.
-        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
         """
         session = self._get_session(session)
         version = utils.pick_microversion(session, '1.37')
@@ -1008,7 +1013,7 @@ class Node(_common.ListMixin, resource.Resource):
         :returns: The HTTP response.
         """
         session = self._get_session(session)
-        version = self._get_microversion_for(session, 'commit')
+        version = self._get_microversion(session, action='commit')
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'vendor_passthru?method={}'
                                     .format(method))
@@ -1033,7 +1038,7 @@ class Node(_common.ListMixin, resource.Resource):
         :returns: The HTTP response.
         """
         session = self._get_session(session)
-        version = self._get_microversion_for(session, 'fetch')
+        version = self._get_microversion(session, action='fetch')
         request = self._prepare_request(requires_id=True)
         request.url = utils.urljoin(request.url, 'vendor_passthru/methods')
 
