@@ -13,52 +13,55 @@
 # import types so that we can reference ListType in sphinx param declarations.
 # We can't just use list, because sphinx gets confused by
 # openstack.resource.Resource.list and openstack.resource2.Resource.list
+
+import contextlib
+import sys
 import types  # noqa
 import warnings
 
 import jsonpatch
 
-from openstack.cloud import _normalize
-from openstack.cloud import _utils
 from openstack.cloud import exc
-from openstack import utils
 
 
-class BaremetalCloudMixin(_normalize.Normalizer):
+def _normalize_port_list(nics):
+    ports = []
+    for row in nics:
+        if isinstance(row, str):
+            address = row
+            row = {}
+        elif 'mac' in row:
+            address = row.pop('mac')
+        else:
+            try:
+                address = row.pop('address')
+            except KeyError:
+                raise TypeError(
+                    "Either 'address' or 'mac' must be provided "
+                    "for port %s" % row)
+        ports.append(dict(row, address=address))
+    return ports
 
-    @property
-    def _baremetal_client(self):
-        if 'baremetal' not in self._raw_clients:
-            client = self._get_raw_client('baremetal')
-            # Do this to force version discovery. We need to do that, because
-            # the endpoint-override trick we do for neutron because
-            # ironicclient just appends a /v1 won't work and will break
-            # keystoneauth - because ironic's versioned discovery endpoint
-            # is non-compliant and doesn't return an actual version dict.
-            client = self._get_versioned_client(
-                'baremetal', min_version=1, max_version='1.latest')
-            self._raw_clients['baremetal'] = client
-        return self._raw_clients['baremetal']
+
+class BaremetalCloudMixin:
 
     def list_nics(self):
         """Return a list of all bare metal ports."""
-        return [nic._to_munch() for nic in self.baremetal.ports(details=True)]
+        return list(self.baremetal.ports(details=True))
 
     def list_nics_for_machine(self, uuid):
         """Returns a list of ports present on the machine node.
 
-        :param uuid: String representing machine UUID value in
-                     order to identify the machine.
+        :param uuid: String representing machine UUID value in order to
+            identify the machine.
         :returns: A list of ports.
         """
         # TODO(dtantsur): support node names here.
-        return [nic._to_munch()
-                for nic in self.baremetal.ports(details=True, node_id=uuid)]
+        return list(self.baremetal.ports(details=True, node_id=uuid))
 
     def get_nic_by_mac(self, mac):
         """Get bare metal NIC by its hardware address (usually MAC)."""
-        results = [nic._to_munch()
-                   for nic in self.baremetal.ports(address=mac, details=True)]
+        results = list(self.baremetal.ports(address=mac, details=True))
         try:
             return results[0]
         except IndexError:
@@ -67,10 +70,9 @@ class BaremetalCloudMixin(_normalize.Normalizer):
     def list_machines(self):
         """List Machines.
 
-        :returns: list of ``munch.Munch`` representing machines.
+        :returns: list of :class:`~openstack.baremetal.v1.node.Node`.
         """
-        return [self._normalize_machine(node)
-                for node in self.baremetal.nodes()]
+        return list(self.baremetal.nodes())
 
     def get_machine(self, name_or_id):
         """Get Machine by name or uuid
@@ -80,11 +82,11 @@ class BaremetalCloudMixin(_normalize.Normalizer):
 
         :param name_or_id: A node name or UUID that will be looked up.
 
-        :returns: ``munch.Munch`` representing the node found or None if no
-                  nodes are found.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: The node found or None if no nodes are found.
         """
         try:
-            return self._normalize_machine(self.baremetal.get_node(name_or_id))
+            return self.baremetal.find_node(name_or_id, ignore_missing=False)
         except exc.OpenStackCloudResourceNotFound:
             return None
 
@@ -93,8 +95,8 @@ class BaremetalCloudMixin(_normalize.Normalizer):
 
         :param mac: Port MAC address to query in order to return a node.
 
-        :returns: ``munch.Munch`` representing the node found or None
-                  if the node is not found.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: The node found or None if no nodes are found.
         """
         nic = self.get_nic_by_mac(mac)
         if nic is None:
@@ -109,16 +111,16 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         metadata about the baremetal machine.
 
         :param name_or_id: String representing machine name or UUID value in
-                           order to identify the machine.
+            order to identify the machine.
 
         :param wait: Boolean value controlling if the method is to wait for
-                     the desired state to be reached or a failure to occur.
+            the desired state to be reached or a failure to occur.
 
-        :param timeout: Integer value, defautling to 3600 seconds, for the$
-                        wait state to reach completion.
+        :param timeout: Integer value, defautling to 3600 seconds, for the
+            wait state to reach completion.
 
-        :returns: ``munch.Munch`` representing the current state of the machine
-                  upon exit of the method.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: Current state of the node.
         """
 
         return_to_available = False
@@ -159,10 +161,26 @@ class BaremetalCloudMixin(_normalize.Normalizer):
                                                            wait=True,
                                                            timeout=timeout)
 
-        return self._normalize_machine(node)
+        return node
+
+    @contextlib.contextmanager
+    def _delete_node_on_error(self, node):
+        try:
+            yield
+        except Exception as exc:
+            self.log.debug("cleaning up node %s because of an error: %s",
+                           node.id, exc)
+            tb = sys.exc_info()[2]
+            try:
+                self.baremetal.delete_node(node)
+            except Exception:
+                self.log.debug("could not remove node %s", node.id,
+                               exc_info=True)
+            raise exc.with_traceback(tb)
 
     def register_machine(self, nics, wait=False, timeout=3600,
-                         lock_timeout=600, **kwargs):
+                         lock_timeout=600, provision_state='available',
+                         **kwargs):
         """Register Baremetal with Ironic
 
         Allows for the registration of Baremetal nodes with Ironic
@@ -176,154 +194,84 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         created are deleted, and the node is removed from Ironic.
 
         :param nics:
-           An array of MAC addresses that represent the
-           network interfaces for the node to be created.
+            An array of ports that represent the network interfaces for the
+            node to be created. The ports are created after the node is
+            enrolled but before it goes through cleaning.
 
-           Example::
+            Example::
 
-              [
-                  {'mac': 'aa:bb:cc:dd:ee:01'},
-                  {'mac': 'aa:bb:cc:dd:ee:02'}
-              ]
+                [
+                    {'address': 'aa:bb:cc:dd:ee:01'},
+                    {'address': 'aa:bb:cc:dd:ee:02'}
+                ]
 
-        :param wait: Boolean value, defaulting to false, to wait for the
-                     node to reach the available state where the node can be
-                     provisioned. It must be noted, when set to false, the
-                     method will still wait for locks to clear before sending
-                     the next required command.
+            Alternatively, you can provide an array of MAC addresses.
 
-        :param timeout: Integer value, defautling to 3600 seconds, for the
-                        wait state to reach completion.
+        :param wait: Boolean value, defaulting to false, to wait for the node
+            to reach the available state where the node can be provisioned. It
+            must be noted, when set to false, the method will still wait for
+            locks to clear before sending the next required command.
+
+        :param timeout: Integer value, defautling to 3600 seconds, for the wait
+            state to reach completion.
 
         :param lock_timeout: Integer value, defaulting to 600 seconds, for
-                             locks to clear.
+            locks to clear.
+
+        :param provision_state: The expected provision state, one of "enroll"
+            "manageable" or "available". Using "available" results in automated
+            cleaning.
 
         :param kwargs: Key value pairs to be passed to the Ironic API,
-                       including uuid, name, chassis_uuid, driver_info,
-                       parameters.
+            including uuid, name, chassis_uuid, driver_info, properties.
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: Returns a ``munch.Munch`` representing the new
-                  baremetal node.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
+        :returns: Current state of the node.
         """
+        if provision_state not in ('enroll', 'manageable', 'available'):
+            raise ValueError('Initial provision state must be enroll, '
+                             'manageable or available, got %s'
+                             % provision_state)
 
-        msg = ("Baremetal machine node failed to be created.")
-        port_msg = ("Baremetal machine port failed to be created.")
+        # Available is tricky: it cannot be directly requested on newer API
+        # versions, we need to go through cleaning. But we cannot go through
+        # cleaning until we create ports.
+        if provision_state != 'available':
+            kwargs['provision_state'] = 'enroll'
+        machine = self.baremetal.create_node(**kwargs)
 
-        url = '/nodes'
-        # TODO(TheJulia): At some point we need to figure out how to
-        # handle data across when the requestor is defining newer items
-        # with the older api.
-        machine = self._baremetal_client.post(url,
-                                              json=kwargs,
-                                              error_message=msg,
-                                              microversion="1.6")
+        with self._delete_node_on_error(machine):
+            # Making a node at least manageable
+            if (machine.provision_state == 'enroll'
+                    and provision_state != 'enroll'):
+                machine = self.baremetal.set_node_provision_state(
+                    machine, 'manage', wait=True, timeout=timeout)
+                machine = self.baremetal.wait_for_node_reservation(
+                    machine, timeout=lock_timeout)
 
-        created_nics = []
-        try:
-            for row in nics:
-                payload = {'address': row['mac'],
-                           'node_uuid': machine['uuid']}
-                nic = self._baremetal_client.post('/ports',
-                                                  json=payload,
-                                                  error_message=port_msg)
-                created_nics.append(nic['uuid'])
-
-        except Exception as e:
-            self.log.debug("ironic NIC registration failed", exc_info=True)
-            # TODO(mordred) Handle failures here
+            # Create NICs before trying to run cleaning
+            created_nics = []
             try:
+                for port in _normalize_port_list(nics):
+                    nic = self.baremetal.create_port(node_id=machine.id,
+                                                     **port)
+                    created_nics.append(nic.id)
+
+            except Exception:
                 for uuid in created_nics:
                     try:
-                        port_url = '/ports/{uuid}'.format(uuid=uuid)
-                        # NOTE(TheJulia): Added in hope that it is logged.
-                        port_msg = ('Failed to delete port {port} for node '
-                                    '{node}').format(port=uuid,
-                                                     node=machine['uuid'])
-                        self._baremetal_client.delete(port_url,
-                                                      error_message=port_msg)
+                        self.baremetal.delete_port(uuid)
                     except Exception:
                         pass
-            finally:
-                version = "1.6"
-                msg = "Baremetal machine failed to be deleted."
-                url = '/nodes/{node_id}'.format(
-                    node_id=machine['uuid'])
-                self._baremetal_client.delete(url,
-                                              error_message=msg,
-                                              microversion=version)
-            raise exc.OpenStackCloudException(
-                "Error registering NICs with the baremetal service: %s"
-                % str(e))
+                raise
 
-        with _utils.shade_exceptions(
-                "Error transitioning node to available state"):
-            if wait:
-                for count in utils.iterate_timeout(
-                        timeout,
-                        "Timeout waiting for node transition to "
-                        "available state"):
+            if (machine.provision_state != 'available'
+                    and provision_state == 'available'):
+                machine = self.baremetal.set_node_provision_state(
+                    machine, 'provide', wait=wait, timeout=timeout)
 
-                    machine = self.get_machine(machine['uuid'])
-
-                    # Note(TheJulia): Per the Ironic state code, a node
-                    # that fails returns to enroll state, which means a failed
-                    # node cannot be determined at this point in time.
-                    if machine['provision_state'] in ['enroll']:
-                        self.node_set_provision_state(
-                            machine['uuid'], 'manage')
-                    elif machine['provision_state'] in ['manageable']:
-                        self.node_set_provision_state(
-                            machine['uuid'], 'provide')
-                    elif machine['last_error'] is not None:
-                        raise exc.OpenStackCloudException(
-                            "Machine encountered a failure: %s"
-                            % machine['last_error'])
-
-                    # Note(TheJulia): Earlier versions of Ironic default to
-                    # None and later versions default to available up until
-                    # the introduction of enroll state.
-                    # Note(TheJulia): The node will transition through
-                    # cleaning if it is enabled, and we will wait for
-                    # completion.
-                    elif machine['provision_state'] in ['available', None]:
-                        break
-
-            else:
-                if machine['provision_state'] in ['enroll']:
-                    self.node_set_provision_state(machine['uuid'], 'manage')
-                    # Note(TheJulia): We need to wait for the lock to clear
-                    # before we attempt to set the machine into provide state
-                    # which allows for the transition to available.
-                    for count in utils.iterate_timeout(
-                            lock_timeout,
-                            "Timeout waiting for reservation to clear "
-                            "before setting provide state"):
-                        machine = self.get_machine(machine['uuid'])
-                        if (machine['reservation'] is None
-                                and machine['provision_state'] != 'enroll'):
-                            # NOTE(TheJulia): In this case, the node has
-                            # has moved on from the previous state and is
-                            # likely not being verified, as no lock is
-                            # present on the node.
-                            self.node_set_provision_state(
-                                machine['uuid'], 'provide')
-                            machine = self.get_machine(machine['uuid'])
-                            break
-
-                        elif machine['provision_state'] in [
-                                'cleaning',
-                                'available']:
-                            break
-
-                        elif machine['last_error'] is not None:
-                            raise exc.OpenStackCloudException(
-                                "Machine encountered a failure: %s"
-                                % machine['last_error'])
-        if not isinstance(machine, str):
-            return self._normalize_machine(machine)
-        else:
             return machine
 
     def unregister_machine(self, nics, uuid, wait=None, timeout=600):
@@ -333,14 +281,14 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         from an Ironic API
 
         :param nics: An array of strings that consist of MAC addresses
-                          to be removed.
+            to be removed.
         :param string uuid: The UUID of the node to be deleted.
 
         :param wait: DEPRECATED, do not use.
 
         :param timeout: Integer value, representing seconds with a default
-                        value of 600, which controls the maximum amount of
-                        time to block until a lock is released on machine.
+            value of 600, which controls the maximum amount of time to block
+            until a lock is released on machine.
 
         :raises: OpenStackCloudException on operation failure.
         """
@@ -366,9 +314,9 @@ class BaremetalCloudMixin(_normalize.Normalizer):
                 "Error unregistering node '%s': Exception occured while"
                 " waiting to be able to proceed: %s" % (machine['uuid'], e))
 
-        for nic in nics:
+        for nic in _normalize_port_list(nics):
             try:
-                port = next(self.baremetal.ports(address=nic['mac']))
+                port = next(self.baremetal.ports(address=nic['address']))
             except StopIteration:
                 continue
             self.baremetal.delete_port(port.id)
@@ -383,34 +331,34 @@ class BaremetalCloudMixin(_normalize.Normalizer):
 
         :param string name_or_id: A machine name or UUID to be updated.
         :param patch:
-           The JSON Patch document is a list of dictonary objects
-           that comply with RFC 6902 which can be found at
-           https://tools.ietf.org/html/rfc6902.
+            The JSON Patch document is a list of dictonary objects that comply
+            with RFC 6902 which can be found at
+            https://tools.ietf.org/html/rfc6902.
 
-           Example patch construction::
+            Example patch construction::
 
-               patch=[]
-               patch.append({
-                   'op': 'remove',
-                   'path': '/instance_info'
-               })
-               patch.append({
-                   'op': 'replace',
-                   'path': '/name',
-                   'value': 'newname'
-               })
-               patch.append({
-                   'op': 'add',
-                   'path': '/driver_info/username',
-                   'value': 'administrator'
-               })
+                patch=[]
+                patch.append({
+                    'op': 'remove',
+                    'path': '/instance_info'
+                })
+                patch.append({
+                    'op': 'replace',
+                    'path': '/name',
+                    'value': 'newname'
+                })
+                patch.append({
+                    'op': 'add',
+                    'path': '/driver_info/username',
+                    'value': 'administrator'
+                })
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: ``munch.Munch`` representing the newly updated node.
+        :returns: Current state of the node.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
         """
-        return self._normalize_machine(
-            self.baremetal.patch_node(name_or_id, patch))
+        return self.baremetal.patch_node(name_or_id, patch)
 
     def update_machine(self, name_or_id, **attrs):
         """Update a machine with new configuration information
@@ -423,20 +371,22 @@ class BaremetalCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: ``munch.Munch`` containing a machine sub-dictonary consisting
-                  of the updated data returned from the API update operation,
-                  and a list named changes which contains all of the API paths
-                  that received updates.
+        :returns: Dictionary containing a machine sub-dictonary consisting
+            of the updated data returned from the API update operation, and a
+            list named changes which contains all of the API paths that
+            received updates.
         """
         machine = self.get_machine(name_or_id)
         if not machine:
             raise exc.OpenStackCloudException(
                 "Machine update failed to find Machine: %s. " % name_or_id)
 
-        new_config = dict(machine, **attrs)
+        new_config = dict(machine._to_munch(), **attrs)
 
         try:
-            patch = jsonpatch.JsonPatch.from_diff(machine, new_config)
+            patch = jsonpatch.JsonPatch.from_diff(
+                machine._to_munch(),
+                new_config)
         except Exception as e:
             raise exc.OpenStackCloudException(
                 "Machine update failed - Error generating JSON patch object "
@@ -452,7 +402,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         change_list = [change['path'] for change in patch]
         node = self.baremetal.update_node(machine, **attrs)
         return dict(
-            node=self._normalize_machine(node),
+            node=node,
             changes=change_list
         )
 
@@ -494,10 +444,9 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         """Validate parameters of the machine.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
+            baremetal node.
         :param bool for_deploy: If ``True``, validate readiness for deployment,
-                                otherwise validate only the power management
-                                properties.
+            otherwise validate only the power management properties.
         :raises: :exc:`~openstack.exceptions.ValidationException`
         """
         if for_deploy:
@@ -523,32 +472,29 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         config drive to be utilized.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
-        :param string state: The desired provision state for the
-                             baremetal node.
+            baremetal node.
+        :param string state: The desired provision state for the baremetal
+            node.
         :param string configdrive: An optional URL or file or path
-                                   representing the configdrive. In the
-                                   case of a directory, the client API
-                                   will create a properly formatted
-                                   configuration drive file and post the
-                                   file contents to the API for
-                                   deployment.
+            representing the configdrive. In the case of a directory, the
+            client API will create a properly formatted configuration drive
+            file and post the file contents to the API for deployment.
         :param boolean wait: A boolean value, defaulted to false, to control
-                             if the method will wait for the desire end state
-                             to be reached before returning.
+            if the method will wait for the desire end state to be reached
+            before returning.
         :param integer timeout: Integer value, defaulting to 3600 seconds,
-                                representing the amount of time to wait for
-                                the desire end state to be reached.
+            representing the amount of time to wait for the desire end state to
+            be reached.
 
         :raises: OpenStackCloudException on operation error.
 
-        :returns: ``munch.Munch`` representing the current state of the machine
-                  upon exit of the method.
+        :returns: Current state of the machine upon exit of the method.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`.
         """
         node = self.baremetal.set_node_provision_state(
             name_or_id, target=state, config_drive=configdrive,
             wait=wait, timeout=timeout)
-        return self._normalize_machine(node)
+        return node
 
     def set_machine_maintenance_state(
             self,
@@ -560,14 +506,13 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         Sets Baremetal maintenance state and maintenance reason.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
-        :param boolean state: The desired state of the node.  True being in
-                              maintenance where as False means the machine
-                              is not in maintenance mode.  This value
-                              defaults to True if not explicitly set.
+            baremetal node.
+        :param boolean state: The desired state of the node. True being in
+            maintenance where as False means the machine is not in maintenance
+            mode.  This value defaults to True if not explicitly set.
         :param string reason: An optional freeform string that is supplied to
-                              the baremetal API to allow for notation as to why
-                              the node is in maintenance state.
+            the baremetal API to allow for notation as to why the node is in
+            maintenance state.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -581,13 +526,13 @@ class BaremetalCloudMixin(_normalize.Normalizer):
     def remove_machine_from_maintenance(self, name_or_id):
         """Remove Baremetal Machine from Maintenance State
 
-        Similarly to set_machine_maintenance_state, this method
-        removes a machine from maintenance state.  It must be noted
-        that this method simpily calls set_machine_maintenace_state
-        for the name_or_id requested and sets the state to False.
+        Similarly to set_machine_maintenance_state, this method removes a
+        machine from maintenance state.  It must be noted that this method
+        simpily calls set_machine_maintenace_state for the name_or_id requested
+        and sets the state to False.
 
         :param string name_or_id: The Name or UUID value representing the
-                                  baremetal node.
+            baremetal node.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -601,8 +546,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         This is a method that sets the node power state to "on".
 
         :params string name_or_id: A string representing the baremetal
-                                   node to have power turned to an "on"
-                                   state.
+            node to have power turned to an "on" state.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -616,8 +560,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         This is a method that sets the node power state to "off".
 
         :params string name_or_id: A string representing the baremetal
-                                   node to have power turned to an "off"
-                                   state.
+            node to have power turned to an "off" state.
 
         :raises: OpenStackCloudException on operation error.
 
@@ -633,8 +576,7 @@ class BaremetalCloudMixin(_normalize.Normalizer):
         to "on".
 
         :params string name_or_id: A string representing the baremetal
-                                   node to have power turned to an "off"
-                                   state.
+            node to have power turned to an "off" state.
 
         :raises: OpenStackCloudException on operation error.
 
