@@ -35,7 +35,7 @@ class User(resource.Resource):
     password = resource.Body('password')
 
     def _prepare_request(self, requires_id=True, prepend_key=True,
-                         base_path=None):
+                         base_path=None, **kwargs):
         """Prepare a request for the database service's create call
 
         User.create calls require the resources_key.
@@ -43,7 +43,10 @@ class User(resource.Resource):
         """
         body = {self.resources_key: self._body.dirty}
 
-        uri = self.base_path % self._uri.attributes
+        if base_path is None:
+            base_path = self.base_path
+
+        uri = base_path % self._uri.attributes
         uri = utils.urljoin(uri, self.id)
 
         return resource._Request(uri, body, None)
