@@ -20,6 +20,14 @@ from openstack import proxy
 
 
 class Proxy(proxy.Proxy):
+    _resource_registry = {
+        "floating_ip": _fip.FloatingIP,
+        "recordset": _rs.Recordset,
+        "zone": _zone.Zone,
+        "zone_export": _zone_export.ZoneExport,
+        "zone_import": _zone_import.ZoneImport,
+        "zone_transfer_request": _zone_transfer.ZoneTransferRequest,
+    }
 
     # ======== Zones ========
     def zones(self, **query):
@@ -55,7 +63,7 @@ class Proxy(proxy.Proxy):
         """Get a zone
 
         :param zone: The value can be the ID of a zone
-             or a :class:`~openstack.dns.v2.zone.Zone` instance.
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :returns: Zone instance.
         :rtype: :class:`~openstack.dns.v2.zone.Zone`
         """
@@ -65,7 +73,7 @@ class Proxy(proxy.Proxy):
         """Delete a zone
 
         :param zone: The value can be the ID of a zone
-             or a :class:`~openstack.dns.v2.zone.Zone` instance.
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.ResourceNotFound` will be raised when
             the zone does not exist.
@@ -108,7 +116,7 @@ class Proxy(proxy.Proxy):
         """Abandon Zone
 
         :param zone: The value can be the ID of a zone to be abandoned
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
 
         :returns: None
         """
@@ -120,7 +128,7 @@ class Proxy(proxy.Proxy):
         """Trigger update of secondary Zone
 
         :param zone: The value can be the ID of a zone to be abandoned
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
 
         :returns: None
         """
@@ -132,9 +140,9 @@ class Proxy(proxy.Proxy):
         """Retrieve a generator of recordsets
 
         :param zone: The optional value can be the ID of a zone
-             or a :class:`~openstack.dns.v2.zone.Zone` instance. If it is not
-             given all recordsets for all zones of the tenant would be
-             retrieved
+            or a :class:`~openstack.dns.v2.zone.Zone` instance. If it is not
+            given all recordsets for all zones of the tenant would be
+            retrieved
         :param dict query: Optional query parameters to be sent to limit the
             resources being returned.
 
@@ -185,9 +193,9 @@ class Proxy(proxy.Proxy):
         """Get a recordset
 
         :param zone: The value can be the ID of a zone
-             or a :class:`~openstack.dns.v2.zone.Zone` instance.
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param recordset: The value can be the ID of a recordset
-             or a :class:`~openstack.dns.v2.recordset.Recordset` instance.
+            or a :class:`~openstack.dns.v2.recordset.Recordset` instance.
         :returns: Recordset instance
         :rtype: :class:`~openstack.dns.v2.recordset.Recordset`
         """
@@ -198,10 +206,10 @@ class Proxy(proxy.Proxy):
         """Delete a zone
 
         :param recordset: The value can be the ID of a recordset
-             or a :class:`~openstack.dns.v2.recordset.Recordset`
-             instance.
+            or a :class:`~openstack.dns.v2.recordset.Recordset`
+            instance.
         :param zone: The value can be the ID of a zone
-             or a :class:`~openstack.dns.v2.zone.Zone` instance.
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.ResourceNotFound` will be raised when
             the zone does not exist. When set to ``True``, no exception will
@@ -221,7 +229,7 @@ class Proxy(proxy.Proxy):
         """Find a single recordset
 
         :param zone: The value can be the ID of a zone
-             or a :class:`~openstack.dns.v2.zone.Zone` instance.
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param name_or_id: The name or ID of a zone
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.ResourceNotFound` will be raised
@@ -268,7 +276,7 @@ class Proxy(proxy.Proxy):
         """Get a zone import record
 
         :param zone: The value can be the ID of a zone import
-             or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
+            or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
         :returns: ZoneImport instance.
         :rtype: :class:`~openstack.dns.v2.zone_import.ZoneImport`
         """
@@ -278,7 +286,7 @@ class Proxy(proxy.Proxy):
         """Delete a zone import
 
         :param zone_import: The value can be the ID of a zone import
-             or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
+            or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.ResourceNotFound` will be raised when
             the zone does not exist.
@@ -310,7 +318,7 @@ class Proxy(proxy.Proxy):
         """Create a new zone export from attributes
 
         :param zone: The value can be the ID of a zone to be exported
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :param dict attrs: Keyword arguments which will be used to create
             a :class:`~openstack.dns.v2.zone_export.ZoneExport`,
             comprised of the properties on the ZoneExport class.
@@ -328,7 +336,7 @@ class Proxy(proxy.Proxy):
         """Get a zone export record
 
         :param zone: The value can be the ID of a zone import
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :returns: ZoneExport instance.
         :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
         """
@@ -338,7 +346,7 @@ class Proxy(proxy.Proxy):
         """Get a zone export record as text
 
         :param zone: The value can be the ID of a zone import
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :returns: ZoneExport instance.
         :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
         """
@@ -349,7 +357,7 @@ class Proxy(proxy.Proxy):
         """Delete a zone export
 
         :param zone_export: The value can be the ID of a zone import
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.ResourceNotFound` will be raised when
             the zone does not exist.
@@ -383,8 +391,8 @@ class Proxy(proxy.Proxy):
         """Get a Floating IP
 
         :param floating_ip: The value can be the ID of a floating ip
-             or a :class:`~openstack.dns.v2.floating_ip.FloatingIP` instance.
-             The ID is in format "region_name:floatingip_id"
+            or a :class:`~openstack.dns.v2.floating_ip.FloatingIP` instance.
+            The ID is in format "region_name:floatingip_id"
         :returns: FloatingIP instance.
         :rtype: :class:`~openstack.dns.v2.floating_ip.FloatingIP`
         """
@@ -402,6 +410,17 @@ class Proxy(proxy.Proxy):
         """
         return self._update(_fip.FloatingIP, floating_ip, **attrs)
 
+    def unset_floating_ip(self, floating_ip):
+        """Unset a Floating IP PTR record
+        :param floating_ip: ID for the floatingip associated with the
+            project.
+        :returns: FloatingIP PTR record.
+        :rtype: :class:`~openstack.dns.v2.fip.FloatipgIP`
+        """
+        # concat `region:floating_ip_id` as id
+        attrs = {'ptrdname': None}
+        return self._update(_fip.FloatingIP, floating_ip, **attrs)
+
     # ======== Zone Transfer ========
     def zone_transfer_requests(self, **query):
         """Retrieve a generator of zone transfer requests
@@ -421,8 +440,8 @@ class Proxy(proxy.Proxy):
         """Get a ZoneTransfer Request info
 
         :param request: The value can be the ID of a transfer request
-             or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
-             instance.
+            or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
+            instance.
         :returns: Zone transfer request instance.
         :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
         """
@@ -432,7 +451,7 @@ class Proxy(proxy.Proxy):
         """Create a new ZoneTransfer Request from attributes
 
         :param zone: The value can be the ID of a zone to be transferred
-             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
+            or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :param dict attrs: Keyword arguments which will be used to create
             a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`,
             comprised of the properties on the ZoneTransferRequest class.
@@ -464,8 +483,8 @@ class Proxy(proxy.Proxy):
         """Delete a ZoneTransfer Request
 
         :param request: The value can be the ID of a zone transfer request
-             or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
-             instance.
+            or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
+            instance.
         :param bool ignore_missing: When set to ``False``
             :class:`~openstack.exceptions.ResourceNotFound` will be raised when
             the zone does not exist.
@@ -495,8 +514,8 @@ class Proxy(proxy.Proxy):
         """Get a ZoneTransfer Accept info
 
         :param request: The value can be the ID of a transfer accept
-             or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`
-             instance.
+            or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`
+            instance.
         :returns: Zone transfer request instance.
         :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferAccept`
         """
@@ -524,4 +543,24 @@ class Proxy(proxy.Proxy):
     def _service_cleanup(self, dry_run=True, client_status_queue=False,
                          identified_resources=None,
                          filters=None, resource_evaluation_fn=None):
-        pass
+        # Delete all zones
+        for obj in self.zones():
+            self._service_cleanup_del_res(
+                self.delete_zone,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
+        # Unset all floatingIPs
+        # NOTE: FloatingIPs are not cleaned when filters are set
+        for obj in self.floating_ips():
+            self._service_cleanup_del_res(
+                self.unset_floating_ip,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
