@@ -44,9 +44,44 @@ from openstack.identity.v3 import system as _system
 from openstack.identity.v3 import trust as _trust
 from openstack.identity.v3 import user as _user
 from openstack import proxy
+from openstack import utils
 
 
 class Proxy(proxy.Proxy):
+    _resource_registry = {
+        "application_credential":
+            _application_credential.ApplicationCredential,
+        "credential": _credential.Credential,
+        "domain": _domain.Domain,
+        "endpoint": _endpoint.Endpoint,
+        "federation_protocol": _federation_protocol.FederationProtocol,
+        "group": _group.Group,
+        "identity_provider": _identity_provider.IdentityProvider,
+        "limit": _limit.Limit,
+        "mapping": _mapping.Mapping,
+        "policy": _policy.Policy,
+        "project": _project.Project,
+        "region": _region.Region,
+        "registered_limit": _registered_limit.RegisteredLimit,
+        "role": _role.Role,
+        "role_assignment": _role_assignment.RoleAssignment,
+        "role_domain_group_assignment":
+            _role_domain_group_assignment.RoleDomainGroupAssignment,
+        "role_domain_user_assignment":
+            _role_domain_user_assignment.RoleDomainUserAssignment,
+        "role_project_group_assignment":
+            _role_project_group_assignment.RoleProjectGroupAssignment,
+        "role_project_user_assignment":
+            _role_project_user_assignment.RoleProjectUserAssignment,
+        "role_system_group_assignment":
+            _role_system_group_assignment.RoleSystemGroupAssignment,
+        "role_system_user_assignment":
+            _role_system_user_assignment.RoleSystemUserAssignment,
+        "service": _service.Service,
+        "system": _system.System,
+        "trust": _trust.Trust,
+        "user": _user.User,
+    }
 
     def create_credential(self, **attrs):
         """Create a new credential from attributes
@@ -405,6 +440,21 @@ class Proxy(proxy.Proxy):
         group = self._get_resource(_group.Group, group)
         return group.check_user(self, user)
 
+    def group_users(self, group, **attrs):
+        """List users in a group
+
+        :param group: Either the ID of a group or a
+            :class:`~openstack.identity.v3.group.Group` instance.
+        :param attrs: Only password_expires_at can be filter for result.
+
+        :return: List of :class:`~openstack.identity.v3.user.User`
+        """
+        group = self._get_resource(_group.Group, group)
+        base_path = utils.urljoin(
+            group.base_path, group.id, 'users')
+        users = self._list(_user.User, base_path=base_path, **attrs)
+        return users
+
     def create_policy(self, **attrs):
         """Create a new policy from attributes
 
