@@ -17,20 +17,33 @@ from openstack import exceptions
 from openstack.image import _base_proxy
 from openstack.image.v2 import image as _image
 from openstack.image.v2 import member as _member
+from openstack.image.v2 import metadef_namespace as _metadef_namespace
+from openstack.image.v2 import metadef_resource_type as _metadef_resource_type
+from openstack.image.v2 import metadef_schema as _metadef_schema
 from openstack.image.v2 import schema as _schema
-from openstack.image.v2 import task as _task
 from openstack.image.v2 import service_info as _si
+from openstack.image.v2 import task as _task
 from openstack import resource
 from openstack import utils
 
 # Rackspace returns this for intermittent import errors
 _IMAGE_ERROR_396 = "Image cannot be imported. Error code: '396'"
 _INT_PROPERTIES = ('min_disk', 'min_ram', 'size', 'virtual_size')
-_RAW_PROPERTIES = ('protected', 'tags')
+_RAW_PROPERTIES = ('is_protected', 'protected', 'tags')
 
 
 class Proxy(_base_proxy.BaseImageProxy):
+    _resource_registry = {
+        "image": _image.Image,
+        "image_member": _member.Member,
+        "metadef_namespace": _metadef_namespace.MetadefNamespace,
+        "schema": _schema.Schema,
+        "info_import": _si.Import,
+        "info_store": _si.Store,
+        "task": _task.Task
+    }
 
+    # ====== IMAGES ======
     def _create_image(self, **kwargs):
         """Create image resource from attributes
         """
@@ -289,13 +302,8 @@ class Proxy(_base_proxy.BaseImageProxy):
 
         try:
             if not use_import:
-                try:
-                    response = image.upload(self)
-                    exceptions.raise_from_response(response)
-                except Exception:
-                    if not supports_import:
-                        raise
-                    use_import = True
+                response = image.upload(self)
+                exceptions.raise_from_response(response)
             if use_import:
                 image.stage(self)
                 image.import_image(self)
@@ -424,22 +432,22 @@ class Proxy(_base_proxy.BaseImageProxy):
         :ref:`download_image-stream-true`.
 
         :param image: The value can be either the ID of an image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            :class:`~openstack.image.v2.image.Image` instance.
 
         :param bool stream: When ``True``, return a :class:`requests.Response`
-                            instance allowing you to iterate over the
-                            response data stream instead of storing its entire
-                            contents in memory. See
-                            :meth:`requests.Response.iter_content` for more
-                            details. *NOTE*: If you do not consume
-                            the entirety of the response you must explicitly
-                            call :meth:`requests.Response.close` or otherwise
-                            risk inefficiencies with the ``requests``
-                            library's handling of connections.
+            instance allowing you to iterate over the
+            response data stream instead of storing its entire
+            contents in memory. See
+            :meth:`requests.Response.iter_content` for more
+            details. *NOTE*: If you do not consume
+            the entirety of the response you must explicitly
+            call :meth:`requests.Response.close` or otherwise
+            risk inefficiencies with the ``requests``
+            library's handling of connections.
 
 
-                            When ``False``, return the entire
-                            contents of the response.
+            When ``False``, return the entire
+            contents of the response.
         :param output: Either a file object or a path to store data into.
         :param int chunk_size: size in bytes to read from the wire and buffer
             at one time. Defaults to 1024
@@ -455,30 +463,38 @@ class Proxy(_base_proxy.BaseImageProxy):
         return image.download(
             self, stream=stream, output=output, chunk_size=chunk_size)
 
-    def delete_image(self, image, ignore_missing=True):
+    def delete_image(self, image, *, store=None, ignore_missing=True):
         """Delete an image
 
         :param image: The value can be either the ID of an image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            :class:`~openstack.image.v2.image.Image` instance.
+        :param store: The value can be either the ID of a store or a
+            :class:`~openstack.image.v2.service_info.Store` instance that the
+            image is associated with. If specified, the image will only be
+            deleted from the specified store.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the image does not exist.
-                    When set to ``True``, no exception will be set when
-                    attempting to delete a nonexistent image.
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the image does not exist.
+            When set to ``True``, no exception will be set when
+            attempting to delete a nonexistent image.
 
         :returns: ``None``
         """
-        self._delete(_image.Image, image, ignore_missing=ignore_missing)
+        if store:
+            store = self._get_resource(_si.Store, store)
+            store.delete_image(self, image, ignore_missing=ignore_missing)
+        else:
+            self._delete(_image.Image, image, ignore_missing=ignore_missing)
 
     def find_image(self, name_or_id, ignore_missing=True):
         """Find a single image
 
         :param name_or_id: The name or ID of a image.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the resource does not exist.
+            When set to ``True``, None will be returned when
+            attempting to find a nonexistent resource.
         :returns: One :class:`~openstack.image.v2.image.Image` or None
         """
         return self._find(_image.Image, name_or_id,
@@ -488,11 +504,11 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Get a single image
 
         :param image: The value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            :class:`~openstack.image.v2.image.Image` instance.
 
         :returns: One :class:`~openstack.image.v2.image.Image`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         return self._get(_image.Image, image)
 
@@ -500,7 +516,7 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Return a generator of images
 
         :param kwargs query: Optional query parameters to be sent to limit
-                                 the resources being returned.
+            the resources being returned.
 
         :returns: A generator of image objects
         :rtype: :class:`~openstack.image.v2.image.Image`
@@ -511,9 +527,9 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Update a image
 
         :param image: Either the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
-        :attrs kwargs: The attributes to update on the image represented
-                       by ``value``.
+            :class:`~openstack.image.v2.image.Image` instance.
+        :param attrs: The attributes to update on the image represented
+            by ``image``.
 
         :returns: The updated image
         :rtype: :class:`~openstack.image.v2.image.Image`
@@ -524,7 +540,7 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Deactivate an image
 
         :param image: Either the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            :class:`~openstack.image.v2.image.Image` instance.
 
         :returns: None
         """
@@ -532,10 +548,10 @@ class Proxy(_base_proxy.BaseImageProxy):
         image.deactivate(self)
 
     def reactivate_image(self, image):
-        """Deactivate an image
+        """Reactivate an image
 
         :param image: Either the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            :class:`~openstack.image.v2.image.Image` instance.
 
         :returns: None
         """
@@ -546,8 +562,8 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Add a tag to an image
 
         :param image: The value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance
-                      that the member will be created for.
+            :class:`~openstack.image.v2.image.Image` instance
+            that the member will be created for.
         :param str tag: The tag to be added
 
         :returns: None
@@ -559,8 +575,8 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Remove a tag to an image
 
         :param image: The value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance
-                      that the member will be created for.
+            :class:`~openstack.image.v2.image.Image` instance
+            that the member will be created for.
         :param str tag: The tag to be removed
 
         :returns: None
@@ -568,15 +584,16 @@ class Proxy(_base_proxy.BaseImageProxy):
         image = self._get_resource(_image.Image, image)
         image.remove_tag(self, tag)
 
+    # ====== IMAGE MEMBERS ======
     def add_member(self, image, **attrs):
         """Create a new member from attributes
 
         :param image: The value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance
-                      that the member will be created for.
+            :class:`~openstack.image.v2.image.Image` instance
+            that the member will be created for.
         :param dict attrs: Keyword arguments which will be used to create
-                           a :class:`~openstack.image.v2.member.Member`,
-                           comprised of the properties on the Member class.
+            a :class:`~openstack.image.v2.member.Member`,
+            comprised of the properties on the Member class.
 
         :returns: The results of member creation
         :rtype: :class:`~openstack.image.v2.member.Member`
@@ -584,16 +601,18 @@ class Proxy(_base_proxy.BaseImageProxy):
         image_id = resource.Resource._get_id(image)
         return self._create(_member.Member, image_id=image_id, **attrs)
 
-    def remove_member(self, member, image, ignore_missing=True):
+    def remove_member(self, member, image=None, ignore_missing=True):
         """Delete a member
 
         :param member: The value can be either the ID of a member or a
-                       :class:`~openstack.image.v2.member.Member` instance.
+            :class:`~openstack.image.v2.member.Member` instance.
+        :param image: The value can be either the ID of an image or a
+            :class:`~openstack.image.v2.image.Image` instance that the member
+            is part of. This is required if ``member`` is an ID.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the member does not exist.
-                    When set to ``True``, no exception will be set when
-                    attempting to delete a nonexistent member.
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the member does not exist. When set to ``True``, no exception will
+            be set when attempting to delete a nonexistent member.
 
         :returns: ``None``
         """
@@ -607,13 +626,13 @@ class Proxy(_base_proxy.BaseImageProxy):
 
         :param name_or_id: The name or ID of a member.
         :param image: This is the image that the member belongs to,
-                      the value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            the value can be the ID of a image or a
+            :class:`~openstack.image.v2.image.Image` instance.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the resource does not exist.
+            When set to ``True``, None will be returned when
+            attempting to find a nonexistent resource.
         :returns: One :class:`~openstack.image.v2.member.Member` or None
         """
         image_id = resource.Resource._get_id(image)
@@ -624,25 +643,27 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Get a single member on an image
 
         :param member: The value can be the ID of a member or a
-                       :class:`~openstack.image.v2.member.Member` instance.
+            :class:`~openstack.image.v2.member.Member` instance.
         :param image: This is the image that the member belongs to.
-                      The value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            The value can be the ID of a image or a
+            :class:`~openstack.image.v2.image.Image` instance.
         :returns: One :class:`~openstack.image.v2.member.Member`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         member_id = resource.Resource._get_id(member)
         image_id = resource.Resource._get_id(image)
         return self._get(_member.Member, member_id=member_id,
                          image_id=image_id)
 
-    def members(self, image):
+    def members(self, image, **query):
         """Return a generator of members
 
         :param image: This is the image that the member belongs to,
-                      the value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
+            the value can be the ID of a image or a
+            :class:`~openstack.image.v2.image.Image` instance.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
 
         :returns: A generator of member objects
         :rtype: :class:`~openstack.image.v2.member.Member`
@@ -654,12 +675,12 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Update the member of an image
 
         :param member: Either the ID of a member or a
-                       :class:`~openstack.image.v2.member.Member` instance.
+            :class:`~openstack.image.v2.member.Member` instance.
         :param image: This is the image that the member belongs to.
-                      The value can be the ID of a image or a
-                      :class:`~openstack.image.v2.image.Image` instance.
-        :attrs kwargs: The attributes to update on the member represented
-                       by ``value``.
+            The value can be the ID of a image or a
+            :class:`~openstack.image.v2.image.Image` instance.
+        :param attrs: The attributes to update on the member represented
+            by ``member``.
 
         :returns: The updated member
         :rtype: :class:`~openstack.image.v2.member.Member`
@@ -669,12 +690,178 @@ class Proxy(_base_proxy.BaseImageProxy):
         return self._update(_member.Member, member_id=member_id,
                             image_id=image_id, **attrs)
 
+    # ====== METADEF NAMESPACES ======
+    def create_metadef_namespace(self, **attrs):
+        """Create a new metadef namespace from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            comprised of the properties on the MetadefNamespace class.
+
+        :returns: The results of metadef namespace creation
+        :rtype: :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+        """
+        return self._create(_metadef_namespace.MetadefNamespace, **attrs)
+
+    def delete_metadef_namespace(self, metadef_namespace, ignore_missing=True):
+        """Delete a metadef namespace
+
+        :param metadef_namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param bool ignore_missing: When set to ``False``,
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the metadef namespace does not exist.
+        :returns: ``None``
+        """
+        self._delete(
+            _metadef_namespace.MetadefNamespace,
+            metadef_namespace,
+            ignore_missing=ignore_missing,
+        )
+
+    # NOTE(stephenfin): There is no 'find_metadef_namespace' since namespaces
+    # are identified by the namespace name, not an arbitrary UUID, meaning
+    # 'find_metadef_namespace' would be identical to 'get_metadef_namespace'
+
+    def get_metadef_namespace(self, metadef_namespace):
+        """Get a single metadef namespace
+
+        :param metadef_namespace: Either the name of a metadef namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: One
+            :class:`~~openstack.image.v2.metadef_namespace.MetadefNamespace`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        return self._get(
+            _metadef_namespace.MetadefNamespace,
+            metadef_namespace,
+        )
+
+    def metadef_namespaces(self, **query):
+        """Return a generator of metadef namespaces
+
+        :returns: A generator object of metadef namespaces
+        :rtype: :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._list(_metadef_namespace.MetadefNamespace, **query)
+
+    def update_metadef_namespace(self, metadef_namespace, **attrs):
+        """Update a server
+
+        :param metadef_namespace: Either the name of a metadef namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param attrs: The attributes to update on the metadef namespace
+            represented by ``metadef_namespace``.
+
+        :returns: The updated metadef namespace
+        :rtype: :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+        """
+        # rather annoyingly, Glance insists on us providing the 'namespace'
+        # argument, even if we're not changing it...
+        if 'namespace' not in attrs:
+            attrs['namespace'] = resource.Resource._get_id(metadef_namespace)
+
+        return self._update(
+            _metadef_namespace.MetadefNamespace,
+            metadef_namespace,
+            **attrs,
+        )
+
+    # ====== METADEF RESOURCE TYPES ======
+    def metadef_resource_types(self, **query):
+        """Return a generator of metadef resource types
+
+        :return: A generator object of metadef resource types
+        :rtype:
+            :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceType`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                when no resource can be found.
+        """
+        return self._list(_metadef_resource_type.MetadefResourceType, **query)
+
+    # ====== METADEF RESOURCE TYPES ASSOCIATION======
+    def create_metadef_resource_type_association(self,
+                                                 metadef_namespace,
+                                                 **attrs):
+        """Creates a resource type association between a namespace
+            and the resource type specified in the body of the request.
+
+        :param dict attrs: Keyword arguments which will be used to create a
+            :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceTypeAssociation`
+            comprised of the properties on the
+            MetadefResourceTypeAssociation class.
+
+        :returns: The results of metadef resource type association creation
+        :rtype:
+            :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceTypeAssociation`
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._create(
+            _metadef_resource_type.MetadefResourceTypeAssociation,
+            namespace_name=namespace_name,
+            **attrs)
+
+    def delete_metadef_resource_type_association(self,
+                                                 metadef_resource_type,
+                                                 metadef_namespace,
+                                                 ignore_missing=True):
+        """Removes a resource type association in a namespace.
+
+        :param metadef_resource_type: The value can be either the name of
+            a metadef resource type association or an
+            :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceTypeAssociation`
+            instance.
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param bool ignore_missing: When set to ``False``,
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the metadef resource type association does not exist.
+        :returns: ``None``
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        self._delete(
+            _metadef_resource_type.MetadefResourceTypeAssociation,
+            metadef_resource_type,
+            namespace_name=namespace_name,
+            ignore_missing=ignore_missing,
+        )
+
+    def metadef_resource_type_associations(self, metadef_namespace, **query):
+        """Return a generator of metadef resource type associations
+
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :return: A generator object of metadef resource type associations
+        :rtype:
+            :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceTypeAssociation`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                when no resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._list(
+            _metadef_resource_type.MetadefResourceTypeAssociation,
+            namespace_name=namespace_name,
+            **query)
+
+    # ====== SCHEMAS ======
     def get_images_schema(self):
         """Get images schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         return self._get(_schema.Schema, requires_id=False,
                          base_path='/schemas/images')
@@ -684,7 +871,7 @@ class Proxy(_base_proxy.BaseImageProxy):
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         return self._get(_schema.Schema, requires_id=False,
                          base_path='/schemas/image')
@@ -694,7 +881,7 @@ class Proxy(_base_proxy.BaseImageProxy):
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         return self._get(_schema.Schema, requires_id=False,
                          base_path='/schemas/members')
@@ -704,16 +891,137 @@ class Proxy(_base_proxy.BaseImageProxy):
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         return self._get(_schema.Schema, requires_id=False,
                          base_path='/schemas/member')
 
+    def get_tasks_schema(self):
+        """Get image tasks schema
+
+        :returns: One :class:`~openstack.image.v2.schema.Schema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_schema.Schema, requires_id=False,
+                         base_path='/schemas/tasks')
+
+    def get_task_schema(self):
+        """Get image task schema
+
+        :returns: One :class:`~openstack.image.v2.schema.Schema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_schema.Schema, requires_id=False,
+                         base_path='/schemas/task')
+
+    def get_metadef_namespace_schema(self):
+        """Get metadata definition namespace schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/namespace')
+
+    def get_metadef_namespaces_schema(self):
+        """Get metadata definition namespaces schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/namespaces')
+
+    def get_metadef_resource_type_schema(self):
+        """Get metadata definition resource type association schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/resource_type')
+
+    def get_metadef_resource_types_schema(self):
+        """Get metadata definition resource type associations schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/resource_types')
+
+    def get_metadef_object_schema(self):
+        """Get metadata definition object schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/object')
+
+    def get_metadef_objects_schema(self):
+        """Get metadata definition objects schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/objects')
+
+    def get_metadef_property_schema(self):
+        """Get metadata definition property schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/property')
+
+    def get_metadef_properties_schema(self):
+        """Get metadata definition properties schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/properties')
+
+    def get_metadef_tag_schema(self):
+        """Get metadata definition tag schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/tag')
+
+    def get_metadef_tags_schema(self):
+        """Get metadata definition tags schema
+
+        :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        """
+        return self._get(_metadef_schema.MetadefSchema, requires_id=False,
+                         base_path='/schemas/metadefs/tags')
+
+    # ====== TASKS ======
     def tasks(self, **query):
         """Return a generator of tasks
 
         :param kwargs query: Optional query parameters to be sent to limit
-                             the resources being returned.
+            the resources being returned.
 
         :returns: A generator of task objects
         :rtype: :class:`~openstack.image.v2.task.Task`
@@ -724,11 +1032,11 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Get task details
 
         :param task: The value can be the ID of a task or a
-                     :class:`~openstack.image.v2.task.Task` instance.
+            :class:`~openstack.image.v2.task.Task` instance.
 
         :returns: One :class:`~openstack.image.v2.task.Task`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
         return self._get(_task.Task, task)
 
@@ -749,22 +1057,22 @@ class Proxy(_base_proxy.BaseImageProxy):
         """Wait for a task to be in a particular status.
 
         :param task: The resource to wait on to reach the specified status.
-                    The resource must have a ``status`` attribute.
+            The resource must have a ``status`` attribute.
         :type resource: A :class:`~openstack.resource.Resource` object.
         :param status: Desired status.
         :param failures: Statuses that would be interpreted as failures.
         :type failures: :py:class:`list`
         :param interval: Number of seconds to wait before to consecutive
-                         checks. Default to 2.
+            checks. Default to 2.
         :param wait: Maximum number of seconds to wait before the change.
-                     Default to 120.
+            Default to 120.
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-                 to the desired status failed to occur in specified seconds.
+            to the desired status failed to occur in specified seconds.
         :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
-                 has transited to one of the failure statuses.
+            has transited to one of the failure statuses.
         :raises: :class:`~AttributeError` if the resource does not have a
-                ``status`` attribute.
+            ``status`` attribute.
         """
         if failures is None:
             failures = ['failure']
@@ -806,39 +1114,39 @@ class Proxy(_base_proxy.BaseImageProxy):
             self.log.debug('Still waiting for resource %s to reach state %s, '
                            'current state is %s', name, status, new_status)
 
-    def get_tasks_schema(self):
-        """Get image tasks schema
-
-        :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
-        """
-        return self._get(_schema.Schema, requires_id=False,
-                         base_path='/schemas/tasks')
-
-    def get_task_schema(self):
-        """Get image task schema
-
-        :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
-        """
-        return self._get(_schema.Schema, requires_id=False,
-                         base_path='/schemas/task')
-
-    def stores(self, **query):
+    # ====== STORES ======
+    def stores(self, details=False, **query):
         """Return a generator of supported image stores
 
         :returns: A generator of store objects
         :rtype: :class:`~openstack.image.v2.service_info.Store`
         """
+        if details:
+            query['base_path'] = utils.urljoin(_si.Store, 'details')
         return self._list(_si.Store, **query)
 
+    # ====== IMPORTS ======
     def get_import_info(self):
         """Get a info about image constraints
 
         :returns: One :class:`~openstack.image.v2.service_info.Import`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
-        return self._get(_si.Import, require_id=False)
+        return self._get(_si.Import, requires_id=False)
+
+    # ====== UTILS ======
+    def wait_for_delete(self, res, interval=2, wait=120):
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :type resource: A :class:`~openstack.resource.Resource` object.
+        :param interval: Number of seconds to wait before to consecutive
+            checks. Default to 2.
+        :param wait: Maximum number of seconds to wait before the change.
+            Default to 120.
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait)
