@@ -9,13 +9,14 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from openstack.common import tag
 from openstack import exceptions
 from openstack.image import _download
 from openstack import resource
 from openstack import utils
 
 
-class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
+class Image(resource.Resource, tag.TagMixin, _download.DownloadMixin):
     resources_key = 'images'
     base_path = '/images'
 
@@ -33,13 +34,24 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     _store_unknown_attrs_as_properties = True
 
     _query_mapping = resource.QueryParameters(
-        "name", "visibility",
-        "member_status", "owner",
-        "status", "size_min", "size_max",
-        "protected", "is_hidden",
-        "sort_key", "sort_dir", "sort", "tag",
-        "created_at", "updated_at",
-        is_hidden="os_hidden")
+        "id",
+        "name",
+        "visibility",
+        "member_status",
+        "owner",
+        "status",
+        "size_min",
+        "size_max",
+        "protected",
+        "is_hidden",
+        "sort_key",
+        "sort_dir",
+        "sort",
+        "tag",
+        "created_at",
+        "updated_at",
+        is_hidden="os_hidden",
+    )
 
     # NOTE: Do not add "self" support here. If you've used Python before,
     # you know that self, while not being a reserved word, has special
@@ -224,8 +236,9 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     os_type = resource.Body('os_type')
     #: The operating system admin username.
     os_admin_user = resource.Body('os_admin_user')
-    #: If true, QEMU guest agent will be exposed to the instance.
-    hw_qemu_guest_agent = resource.Body('hw_qemu_guest_agent', type=bool)
+    #: A string boolean, which if "true", QEMU guest agent will be exposed
+    #: to the instance.
+    hw_qemu_guest_agent = resource.Body('hw_qemu_guest_agent', type=str)
     #: If true, require quiesce on snapshot via QEMU guest agent.
     os_require_quiesce = resource.Body('os_require_quiesce', type=bool)
     #: The URL for the schema describing a virtual machine image.
@@ -251,15 +264,32 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
         """
         self._action(session, "reactivate")
 
-    def upload(self, session):
-        """Upload data into an existing image"""
+    def upload(self, session, *, data=None):
+        """Upload data into an existing image
+
+        :param session: The session to use for making this request
+        :param data: Optional data to be uploaded. If not provided, the
+            `~Image.data` attribute will be used
+        :returns: The server response
+        """
+        if data:
+            self.data = data
         url = utils.urljoin(self.base_path, self.id, 'file')
         return session.put(url, data=self.data,
                            headers={"Content-Type": "application/octet-stream",
                                     "Accept": ""})
 
-    def stage(self, session):
-        """Stage binary image data into an existing image"""
+    def stage(self, session, *, data=None):
+        """Stage binary image data into an existing image
+
+        :param session: The session to use for making this request
+        :param data: Optional data to be uploaded. If not provided, the
+            `~Image.data` attribute will be used
+        :returns: The server response
+        """
+        if data:
+            self.data = data
+
         url = utils.urljoin(self.base_path, self.id, 'stage')
         response = session.put(
             url, data=self.data,
@@ -269,23 +299,52 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
         return self
 
     def import_image(self, session, method='glance-direct', uri=None,
-                     store=None):
+                     store=None, stores=None, all_stores=None,
+                     all_stores_must_succeed=None):
         """Import Image via interoperable image import process"""
+        if all_stores and (store or stores):
+            raise exceptions.InvalidRequest(
+                "all_stores is mutually exclusive with"
+                " store and stores")
+        if store and stores:
+            raise exceptions.InvalidRequest(
+                "store and stores are mutually exclusive."
+                " Please just use stores.")
+        if store:
+            stores = [store]
+        else:
+            stores = stores or []
+
         url = utils.urljoin(self.base_path, self.id, 'import')
         json = {'method': {'name': method}}
         if uri:
             if method == 'web-download':
                 json['method']['uri'] = uri
-            elif method == 'glance-direct':
-                json['method']['uri'] = uri
             else:
                 raise exceptions.InvalidRequest('URI is only supported with '
                                                 'method: "web-download"')
+        if all_stores is not None:
+            json['all_stores'] = all_stores
+        if all_stores_must_succeed is not None:
+            json['all_stores_must_succeed'] = all_stores_must_succeed
+        for s in stores:
+            json.setdefault('stores', [])
+            json['stores'].append(s.id)
+
         headers = {}
+        # Backward compat
         if store is not None:
             headers = {'X-Image-Meta-Store': store.id}
         session.post(url, json=json, headers=headers)
 
+    def _consume_header_attrs(self, attrs):
+        self.image_import_methods = []
+        _image_import_methods = attrs.pop('OpenStack-image-import-methods', '')
+        if _image_import_methods:
+            self.image_import_methods = _image_import_methods.split(',')
+
+        return super()._consume_header_attrs(attrs)
+
     def _prepare_request(self, requires_id=None, prepend_key=False,
                          patch=False, base_path=None, **kwargs):
         request = super(Image, self)._prepare_request(requires_id=requires_id,
