@@ -18,6 +18,8 @@ class QoSRuleType(resource.Resource):
     resources_key = 'rule_types'
     base_path = '/qos/rule-types'
 
+    _allow_unknown_attrs_in_body = True
+
     # capabilities
     allow_create = False
     allow_fetch = True
@@ -25,7 +27,8 @@ class QoSRuleType(resource.Resource):
     allow_delete = False
     allow_list = True
 
-    _query_mapping = resource.QueryParameters('type', 'drivers')
+    _query_mapping = resource.QueryParameters(
+        'type', 'drivers', 'all_rules', 'all_supported')
 
     # Properties
     #: QoS rule type name.
