@@ -27,6 +27,8 @@ class Quota(resource.Resource):
     allow_list = True
 
     # Properties
+    #: Flag to check the quota usage before setting the new limit. *Type: bool*
+    check_limit = resource.Body('check_limit', type=bool)
     #: The maximum amount of floating IPs you can have. *Type: int*
     floating_ips = resource.Body('floatingip', type=int)
     #: The maximum amount of health monitors you can create. *Type: int*
@@ -114,7 +116,7 @@ class QuotaDetails(Quota):
     #: The maximum amount of ports you can create. *Type: dict*
     ports = resource.Body('port', type=dict)
     #: The ID of the project these quota values are for.
-    project_id = resource.Body('tenant_id', alternate_id=True)
+    project_id = resource.Body('project_id', alternate_id=True)
     #: The maximum amount of RBAC policies you can create. *Type: dict*
     rbac_policies = resource.Body('rbac_policy', type=dict)
     #: The maximum amount of routers you can create. *Type: int*
