@@ -511,7 +511,6 @@ class Resource(dict):
     _uri = None
     _computed = None
     _original_body = None
-    _delete_response_class = None
     _store_unknown_attrs_as_properties = False
     _allow_unknown_attrs_in_body = False
     _unknown_attrs_in_body = None
@@ -628,17 +627,6 @@ class Resource(dict):
             ]
         )
 
-    def warning_if_attribute_deprecated(self, attr, value):
-        if value and self.deprecated:
-            if not self.deprecation_reason:
-                LOG.warning(
-                    "The option [%s] has been deprecated. "
-                    "Please avoid using it.",
-                    attr,
-                )
-            else:
-                LOG.warning(self.deprecation_reason)
-
     def __getattribute__(self, name):
         """Return an attribute on this instance
 
@@ -695,6 +683,9 @@ class Resource(dict):
                         DeprecationWarning,
                     )
                     return getattr(self, attr)
+            if self._allow_unknown_attrs_in_body:
+                if name in self._unknown_attrs_in_body:
+                    return self._unknown_attrs_in_body[name]
         raise KeyError(name)
 
     def __delitem__(self, name):
@@ -1019,6 +1010,34 @@ class Resource(dict):
         """
         return cls(_synchronized=synchronized, connection=connection, **obj)
 
+    def _attr_to_dict(self, attr, to_munch):
+        """For a given attribute, convert it into a form suitable for a dict value.
+
+        :param bool attr: Attribute name to convert
+
+        :return: A dictionary of key/value pairs where keys are named
+            as they exist as attributes of this class.
+        :param bool _to_munch: Converts subresources to munch instead of dict.
+        """
+        value = getattr(self, attr, None)
+        if isinstance(value, Resource):
+            return value.to_dict(_to_munch=to_munch)
+        elif isinstance(value, dict) and to_munch:
+            return munch.Munch(value)
+        elif value and isinstance(value, list):
+            converted = []
+            for raw in value:
+                if isinstance(raw, Resource):
+                    converted.append(
+                        raw.to_dict(_to_munch=to_munch)
+                    )
+                elif isinstance(raw, dict) and to_munch:
+                    converted.append(munch.Munch(raw))
+                else:
+                    converted.append(raw)
+            return converted
+        return value
+
     def to_dict(
         self,
         body=True,
@@ -1067,6 +1086,15 @@ class Resource(dict):
         # isinstance stricly requires this to be a tuple
         components = tuple(components)
 
+        if body and self._allow_unknown_attrs_in_body:
+            for key in self._unknown_attrs_in_body:
+                converted = self._attr_to_dict(
+                    key,
+                    to_munch=_to_munch,
+                )
+                if not ignore_none or converted is not None:
+                    mapping[key] = converted
+
         # NOTE: This is similar to the implementation in _get_mapping
         # but is slightly different in that we're looking at an instance
         # and we're mapping names on this class to their actual stored
@@ -1080,27 +1108,13 @@ class Resource(dict):
                 # Make sure base classes don't end up overwriting
                 # mappings we've found previously in subclasses.
                 if key not in mapping:
-                    value = getattr(self, attr, None)
-                    if ignore_none and value is None:
+                    converted = self._attr_to_dict(
+                        attr,
+                        to_munch=_to_munch,
+                    )
+                    if ignore_none and converted is None:
                         continue
-                    if isinstance(value, Resource):
-                        mapping[key] = value.to_dict(_to_munch=_to_munch)
-                    elif isinstance(value, dict) and _to_munch:
-                        mapping[key] = munch.Munch(value)
-                    elif value and isinstance(value, list):
-                        converted = []
-                        for raw in value:
-                            if isinstance(raw, Resource):
-                                converted.append(
-                                    raw.to_dict(_to_munch=_to_munch)
-                                )
-                            elif isinstance(raw, dict) and _to_munch:
-                                converted.append(munch.Munch(raw))
-                            else:
-                                converted.append(raw)
-                        mapping[key] = converted
-                    else:
-                        mapping[key] = value
+                    mapping[key] = converted
 
         return mapping
 
@@ -1236,7 +1250,9 @@ class Resource(dict):
         """
         if has_body is None:
             has_body = self.has_body
+
         exceptions.raise_from_response(response, error_message=error_message)
+
         if has_body:
             try:
                 body = response.json()
@@ -1300,8 +1316,8 @@ class Resource(dict):
         )
 
     @classmethod
-    def _get_microversion_for_list(cls, session):
-        """Get microversion to use when listing resources.
+    def _get_microversion(cls, session, *, action):
+        """Get microversion to use for the given action.
 
         The base version uses the following logic:
 
@@ -1313,9 +1329,22 @@ class Resource(dict):
 
         Subclasses can override this method if more complex logic is needed.
 
-        :param session: :class`keystoneauth1.adapter.Adapter`
-        :return: microversion as string or ``None``
+        :param session: The session to use for making the request.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param action: One of "fetch", "commit", "create", "delete", "patch".
+        :type action: str
+        :return: Microversion as string or ``None``
         """
+        if action not in {
+            'list',
+            'fetch',
+            'commit',
+            'create',
+            'delete',
+            'patch',
+        }:
+            raise ValueError('Invalid action: %s' % action)
+
         if session.default_microversion:
             return session.default_microversion
 
@@ -1323,28 +1352,13 @@ class Resource(dict):
             session, cls._max_microversion
         )
 
-    def _get_microversion_for(self, session, action):
-        """Get microversion to use for the given action.
-
-        The base version uses :meth:`_get_microversion_for_list`.
-        Subclasses can override this method if more complex logic is needed.
-
-        :param session: :class`keystoneauth1.adapter.Adapter`
-        :param action: One of "fetch", "commit", "create", "delete", "patch".
-            Unused in the base implementation.
-        :return: microversion as string or ``None``
-        """
-        if action not in ('fetch', 'commit', 'create', 'delete', 'patch'):
-            raise ValueError('Invalid action: %s' % action)
-
-        return self._get_microversion_for_list(session)
-
     def _assert_microversion_for(
         self,
         session,
         action,
         expected,
         error_message=None,
+        maximum=None,
     ):
         """Enforce that the microversion for action satisfies the requirement.
 
@@ -1353,6 +1367,7 @@ class Resource(dict):
         :param expected: Expected microversion.
         :param error_message: Optional error message with details. Will be
             prepended to the message generated here.
+        :param maximum: Maximum microversion.
         :return: resulting microversion as string.
         :raises: :exc:`~openstack.exceptions.NotSupported` if the version
             used for the action is lower than the expected one.
@@ -1365,29 +1380,43 @@ class Resource(dict):
 
             raise exceptions.NotSupported(message)
 
-        actual = self._get_microversion_for(session, action)
+        actual = self._get_microversion(session, action=action)
 
-        if expected is None:
-            return actual
-        elif actual is None:
+        if actual is None:
             message = (
                 "API version %s is required, but the default "
                 "version will be used."
             ) % expected
             _raise(message)
-
         actual_n = discover.normalize_version_number(actual)
-        expected_n = discover.normalize_version_number(expected)
-        if actual_n < expected_n:
-            message = (
-                "API version %(expected)s is required, but %(actual)s "
-                "will be used."
-            ) % {'expected': expected, 'actual': actual}
-            _raise(message)
+
+        if expected is not None:
+            expected_n = discover.normalize_version_number(expected)
+            if actual_n < expected_n:
+                message = (
+                    "API version %(expected)s is required, but %(actual)s "
+                    "will be used."
+                ) % {'expected': expected, 'actual': actual}
+                _raise(message)
+        if maximum is not None:
+            maximum_n = discover.normalize_version_number(maximum)
+            # Assume that if a service supports higher versions, it also
+            # supports lower ones. Breaks for services that remove old API
+            # versions (which is not something they should do).
+            if actual_n > maximum_n:
+                return maximum
 
         return actual
 
-    def create(self, session, prepend_key=True, base_path=None, **params):
+    def create(
+        self,
+        session,
+        prepend_key=True,
+        base_path=None,
+        *,
+        microversion=None,
+        **params
+    ):
         """Create a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1397,16 +1426,18 @@ class Resource(dict):
             True.
         :param str base_path: Base part of the URI for creating resources, if
             different from :data:`~openstack.resource.Resource.base_path`.
+        :param str microversion: API version to override the negotiated one.
         :param dict params: Additional params to pass.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
             :data:`Resource.allow_create` is not set to ``True``.
         """
         if not self.allow_create:
-            raise exceptions.MethodNotSupported(self, "create")
+            raise exceptions.MethodNotSupported(self, 'create')
 
         session = self._get_session(session)
-        microversion = self._get_microversion_for(session, 'create')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='create')
         requires_id = (
             self.create_requires_id
             if self.create_requires_id is not None
@@ -1467,6 +1498,8 @@ class Resource(dict):
         data,
         prepend_key=True,
         base_path=None,
+        *,
+        microversion=None,
         **params,
     ):
         """Create multiple remote resources based on this class and data.
@@ -1479,6 +1512,7 @@ class Resource(dict):
             True.
         :param str base_path: Base part of the URI for creating resources, if
             different from :data:`~openstack.resource.Resource.base_path`.
+        :param str microversion: API version to override the negotiated one.
         :param dict params: Additional params to pass.
 
         :return: A generator of :class:`Resource` objects.
@@ -1486,7 +1520,7 @@ class Resource(dict):
             :data:`Resource.allow_create` is not set to ``True``.
         """
         if not cls.allow_create:
-            raise exceptions.MethodNotSupported(cls, "create")
+            raise exceptions.MethodNotSupported(cls, 'create')
 
         if not (
             data
@@ -1496,7 +1530,8 @@ class Resource(dict):
             raise ValueError('Invalid data passed: %s' % data)
 
         session = cls._get_session(session)
-        microversion = cls._get_microversion_for(cls, session, 'create')
+        if microversion is None:
+            microversion = cls._get_microversion(session, action='create')
         requires_id = (
             cls.create_requires_id
             if cls.create_requires_id is not None
@@ -1569,6 +1604,8 @@ class Resource(dict):
         base_path=None,
         error_message=None,
         skip_cache=False,
+        *,
+        microversion=None,
         **params,
     ):
         """Get a remote resource based on this instance.
@@ -1583,6 +1620,7 @@ class Resource(dict):
             requested object does not exist.
         :param bool skip_cache: A boolean indicating whether optional API
             cache should be skipped for this invocation.
+        :param str microversion: API version to override the negotiated one.
         :param dict params: Additional parameters that can be consumed.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
@@ -1591,13 +1629,14 @@ class Resource(dict):
             the resource was not found.
         """
         if not self.allow_fetch:
-            raise exceptions.MethodNotSupported(self, "fetch")
+            raise exceptions.MethodNotSupported(self, 'fetch')
 
         request = self._prepare_request(
-            requires_id=requires_id, base_path=base_path
+            requires_id=requires_id, base_path=base_path,
         )
         session = self._get_session(session)
-        microversion = self._get_microversion_for(session, 'fetch')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='fetch')
         response = session.get(
             request.url,
             microversion=microversion,
@@ -1612,13 +1651,14 @@ class Resource(dict):
         self._translate_response(response, **kwargs)
         return self
 
-    def head(self, session, base_path=None):
+    def head(self, session, base_path=None, *, microversion=None):
         """Get headers from a remote resource based on this instance.
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param str base_path: Base part of the URI for fetching resources, if
             different from :data:`~openstack.resource.Resource.base_path`.
+        :param str microversion: API version to override the negotiated one.
 
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
@@ -1627,12 +1667,13 @@ class Resource(dict):
             was not found.
         """
         if not self.allow_head:
-            raise exceptions.MethodNotSupported(self, "head")
-
-        request = self._prepare_request(base_path=base_path)
+            raise exceptions.MethodNotSupported(self, 'head')
 
         session = self._get_session(session)
-        microversion = self._get_microversion_for(session, 'fetch')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='fetch')
+
+        request = self._prepare_request(base_path=base_path)
         response = session.head(request.url, microversion=microversion)
 
         self.microversion = microversion
@@ -1653,6 +1694,8 @@ class Resource(dict):
         has_body=True,
         retry_on_conflict=None,
         base_path=None,
+        *,
+        microversion=None,
         **kwargs,
     ):
         """Commit the state of the instance to the remote resource.
@@ -1666,6 +1709,7 @@ class Resource(dict):
             CONFLICT (409). Value of ``None`` leaves the `Adapter` defaults.
         :param str base_path: Base part of the URI for modifying resources, if
             different from :data:`~openstack.resource.Resource.base_path`.
+        :param str microversion: API version to override the negotiated one.
         :param dict kwargs: Parameters that will be passed to
             _prepare_request()
 
@@ -1673,6 +1717,9 @@ class Resource(dict):
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
             :data:`Resource.allow_commit` is not set to ``True``.
         """
+        if not self.allow_commit:
+            raise exceptions.MethodNotSupported(self, 'commit')
+
         # The id cannot be dirty for an commit
         self._body._dirty.discard("id")
 
@@ -1680,18 +1727,18 @@ class Resource(dict):
         if not self.requires_commit:
             return self
 
-        if not self.allow_commit:
-            raise exceptions.MethodNotSupported(self, "commit")
-
         # Avoid providing patch unconditionally to avoid breaking subclasses
         # without it.
         if self.commit_jsonpatch:
             kwargs['patch'] = True
 
         request = self._prepare_request(
-            prepend_key=prepend_key, base_path=base_path, **kwargs
+            prepend_key=prepend_key,
+            base_path=base_path,
+            **kwargs,
         )
-        microversion = self._get_microversion_for(session, 'commit')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='commit')
 
         return self._commit(
             session,
@@ -1739,6 +1786,7 @@ class Resource(dict):
 
         self.microversion = microversion
         self._translate_response(response, has_body=has_body)
+
         return self
 
     def _convert_patch(self, patch):
@@ -1777,6 +1825,8 @@ class Resource(dict):
         has_body=True,
         retry_on_conflict=None,
         base_path=None,
+        *,
+        microversion=None,
     ):
         """Patch the remote resource.
 
@@ -1795,11 +1845,15 @@ class Resource(dict):
             CONFLICT (409). Value of ``None`` leaves the `Adapter` defaults.
         :param str base_path: Base part of the URI for modifying resources, if
             different from :data:`~openstack.resource.Resource.base_path`.
+        :param str microversion: API version to override the negotiated one.
 
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
             :data:`Resource.allow_patch` is not set to ``True``.
         """
+        if not self.allow_patch:
+            raise exceptions.MethodNotSupported(self, 'patch')
+
         # The id cannot be dirty for an commit
         self._body._dirty.discard("id")
 
@@ -1807,13 +1861,13 @@ class Resource(dict):
         if not patch and not self.requires_commit:
             return self
 
-        if not self.allow_patch:
-            raise exceptions.MethodNotSupported(self, "patch")
-
         request = self._prepare_request(
-            prepend_key=prepend_key, base_path=base_path, patch=True
+            prepend_key=prepend_key,
+            base_path=base_path,
+            patch=True,
         )
-        microversion = self._get_microversion_for(session, 'patch')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='patch')
         if patch:
             request.body += self._convert_patch(patch)
 
@@ -1826,11 +1880,13 @@ class Resource(dict):
             retry_on_conflict=retry_on_conflict,
         )
 
-    def delete(self, session, error_message=None, **kwargs):
+    def delete(self, session, error_message=None, *, microversion=None,
+               **kwargs):
         """Delete the remote resource based on this instance.
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param str microversion: API version to override the negotiated one.
         :param dict kwargs: Parameters that will be passed to
             _prepare_request()
 
@@ -1841,7 +1897,8 @@ class Resource(dict):
             the resource was not found.
         """
 
-        response = self._raw_delete(session, **kwargs)
+        response = self._raw_delete(session, microversion=microversion,
+                                    **kwargs)
         kwargs = {}
         if error_message:
             kwargs['error_message'] = error_message
@@ -1849,16 +1906,19 @@ class Resource(dict):
         self._translate_response(response, has_body=False, **kwargs)
         return self
 
-    def _raw_delete(self, session, **kwargs):
+    def _raw_delete(self, session, microversion=None, **kwargs):
         if not self.allow_delete:
-            raise exceptions.MethodNotSupported(self, "delete")
+            raise exceptions.MethodNotSupported(self, 'delete')
 
         request = self._prepare_request(**kwargs)
         session = self._get_session(session)
-        microversion = self._get_microversion_for(session, 'delete')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='delete')
 
         return session.delete(
-            request.url, headers=request.headers, microversion=microversion
+            request.url,
+            headers=request.headers,
+            microversion=microversion,
         )
 
     @classmethod
@@ -1868,6 +1928,8 @@ class Resource(dict):
         paginated=True,
         base_path=None,
         allow_unknown_params=False,
+        *,
+        microversion=None,
         **params,
     ):
         """This method is a generator which yields resource objects.
@@ -1887,6 +1949,7 @@ class Resource(dict):
             unknown query parameters. This allows getting list of 'filters' and
             passing everything known to the server. ``False`` will result in
             validation exception when unknown query parameters are passed.
+        :param str microversion: API version to override the negotiated one.
         :param dict params: These keyword arguments are passed through the
             :meth:`~openstack.resource.QueryParamter._transpose` method
             to find if any of them match expected query parameters to be sent
@@ -1895,6 +1958,9 @@ class Resource(dict):
             checked against the :data:`~openstack.resource.Resource.base_path`
             format string to see if any path fragments need to be filled in by
             the contents of this argument.
+            Parameters supported as filters by the server side are passed in
+            the API call, remaining parameters are applied as filters to the
+            retrieved results.
 
         :return: A generator of :class:`Resource` objects.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
@@ -1903,18 +1969,32 @@ class Resource(dict):
             contains invalid params.
         """
         if not cls.allow_list:
-            raise exceptions.MethodNotSupported(cls, "list")
+            raise exceptions.MethodNotSupported(cls, 'list')
+
         session = cls._get_session(session)
-        microversion = cls._get_microversion_for_list(session)
+        if microversion is None:
+            microversion = cls._get_microversion(session, action='list')
 
         if base_path is None:
             base_path = cls.base_path
-        params = cls._query_mapping._validate(
+        api_filters = cls._query_mapping._validate(
             params,
             base_path=base_path,
-            allow_unknown_params=allow_unknown_params,
+            allow_unknown_params=True,
         )
-        query_params = cls._query_mapping._transpose(params, cls)
+        client_filters = dict()
+        # Gather query parameters which are not supported by the server
+        for (k, v) in params.items():
+            if (
+                # Known attr
+                hasattr(cls, k)
+                # Is real attr property
+                and isinstance(getattr(cls, k), Body)
+                # not included in the query_params
+                and k not in cls._query_mapping._mapping.keys()
+            ):
+                client_filters[k] = v
+        query_params = cls._query_mapping._transpose(api_filters, cls)
         uri = base_path % params
         uri_params = {}
 
@@ -1925,6 +2005,18 @@ class Resource(dict):
             if hasattr(cls, k) and isinstance(getattr(cls, k), URI):
                 uri_params[k] = v
 
+        def _dict_filter(f, d):
+            """Dict param based filtering"""
+            if not d:
+                return False
+            for key in f.keys():
+                if isinstance(f[key], dict):
+                    if not _dict_filter(f[key], d.get(key, None)):
+                        return False
+                elif d.get(key, None) != f[key]:
+                    return False
+            return True
+
         # Track the total number of resources yielded so we can paginate
         # swift objects
         total_yielded = 0
@@ -1968,7 +2060,20 @@ class Resource(dict):
                     **raw_resource,
                 )
                 marker = value.id
-                yield value
+                filters_matched = True
+                # Iterate over client filters and return only if matching
+                for key in client_filters.keys():
+                    if isinstance(client_filters[key], dict):
+                        if not _dict_filter(
+                                client_filters[key], value.get(key, None)):
+                            filters_matched = False
+                            break
+                    elif value.get(key, None) != client_filters[key]:
+                        filters_matched = False
+                        break
+
+                if filters_matched:
+                    yield value
                 total_yielded += 1
 
             if resources and paginated:
@@ -1994,34 +2099,40 @@ class Resource(dict):
     def _get_next_link(cls, uri, response, data, marker, limit, total_yielded):
         next_link = None
         params = {}
+
         if isinstance(data, dict):
             pagination_key = cls.pagination_key
 
             if not pagination_key and 'links' in data:
                 # api-wg guidelines are for a links dict in the main body
                 pagination_key = 'links'
+
             if not pagination_key and cls.resources_key:
                 # Nova has a {key}_links dict in the main body
                 pagination_key = '{key}_links'.format(key=cls.resources_key)
+
             if pagination_key:
                 links = data.get(pagination_key, {})
                 # keystone might return a dict
                 if isinstance(links, dict):
                     links = ({k: v} for k, v in links.items())
+
                 for item in links:
                     if item.get('rel') == 'next' and 'href' in item:
                         next_link = item['href']
                         break
+
             # Glance has a next field in the main body
             next_link = next_link or data.get('next')
             if next_link and next_link.startswith('/v'):
-                next_link = next_link[next_link.find('/', 1) + 1 :]
+                next_link = next_link[next_link.find('/', 1):]
 
         if not next_link and 'next' in response.links:
             # RFC5988 specifies Link headers and requests parses them if they
             # are there. We prefer link dicts in resource body, but if those
             # aren't there and Link headers are, use them.
             next_link = response.links['next']['uri']
+
         # Swift provides a count of resources in a header and a list body
         if not next_link and cls.pagination_key:
             total_count = response.headers.get(cls.pagination_key)
@@ -2049,6 +2160,7 @@ class Resource(dict):
             next_link = uri
             params['marker'] = marker
             params['limit'] = limit
+
         return next_link, params
 
     @classmethod
@@ -2078,6 +2190,8 @@ class Resource(dict):
         name_or_id,
         ignore_missing=True,
         list_base_path=None,
+        *,
+        microversion=None,
         **params,
     ):
         """Find a resource by its name or id.
@@ -2092,6 +2206,7 @@ class Resource(dict):
             returned when attempting to find a nonexistent resource.
         :param str list_base_path: base_path to be used when need listing
             resources.
+        :param str microversion: API version to override the negotiated one.
         :param dict params: Any additional parameters to be passed into
             underlying methods, such as to
             :meth:`~openstack.resource.Resource.existing` in order to pass on
@@ -2110,7 +2225,7 @@ class Resource(dict):
             match = cls.existing(
                 id=name_or_id, connection=session._get_connection(), **params
             )
-            return match.fetch(session, **params)
+            return match.fetch(session, microversion=microversion, **params)
         except (exceptions.NotFoundException, exceptions.BadRequestException):
             # NOTE(gtema): There are few places around openstack that return
             # 400 if we try to GET resource and it doesn't exist.
