@@ -20,11 +20,9 @@ Functional tests for `shade` compute methods.
 import datetime
 
 from fixtures import TimeoutException
-import six
 
 from openstack.cloud import exc
 from openstack.tests.functional import base
-from openstack.tests.functional.cloud.util import pick_flavor
 from openstack import utils
 
 
@@ -35,11 +33,6 @@ class TestCompute(base.BaseFunctionalTest):
         self.TIMEOUT_SCALING_FACTOR = 1.5
 
         super(TestCompute, self).setUp()
-        self.flavor = pick_flavor(
-            self.user_cloud.list_flavors(get_extra=False))
-        if self.flavor is None:
-            self.assertFalse('no sensible flavor available')
-        self.image = self.pick_image()
         self.server_name = self.getUniqueString()
 
     def _cleanup_servers_and_volumes(self, server_name):
@@ -77,11 +70,12 @@ class TestCompute(base.BaseFunctionalTest):
             wait=True)
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
-        self.assertEqual(self.flavor.id, server['flavor']['id'])
+        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(self.server_name, wait=True))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_create_and_delete_server_auto_ip_delete_ips(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -93,12 +87,13 @@ class TestCompute(base.BaseFunctionalTest):
             wait=True)
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
-        self.assertEqual(self.flavor.id, server['flavor']['id'])
+        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(
                 self.server_name, wait=True, delete_ips=True))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_attach_detach_volume(self):
         self.skipTest('Volume functional tests temporarily disabled')
@@ -124,12 +119,13 @@ class TestCompute(base.BaseFunctionalTest):
             wait=True)
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
-        self.assertEqual(self.flavor.id, server['flavor']['id'])
+        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
         self.assertTrue(server['has_config_drive'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(self.server_name, wait=True))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_create_and_delete_server_with_config_drive_none(self):
         # check that we're not sending invalid values for config_drive
@@ -144,13 +140,14 @@ class TestCompute(base.BaseFunctionalTest):
             wait=True)
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
-        self.assertEqual(self.flavor.id, server['flavor']['id'])
+        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
         self.assertFalse(server['has_config_drive'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(
                 self.server_name, wait=True))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_list_all_servers(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -183,11 +180,12 @@ class TestCompute(base.BaseFunctionalTest):
             wait=True)
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
-        self.assertEqual(self.flavor.id, server['flavor']['id'])
+        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(self.server_name, wait=True))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_get_server_console(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -200,7 +198,7 @@ class TestCompute(base.BaseFunctionalTest):
         # returning a string tests that the call is correct. Testing that
         # the cloud returns actual data in the output is out of scope.
         log = self.user_cloud._get_server_console_output(server_id=server.id)
-        self.assertTrue(isinstance(log, six.string_types))
+        self.assertTrue(isinstance(log, str))
 
     def test_get_server_console_name_or_id(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -210,7 +208,7 @@ class TestCompute(base.BaseFunctionalTest):
             flavor=self.flavor,
             wait=True)
         log = self.user_cloud.get_server_console(server=self.server_name)
-        self.assertTrue(isinstance(log, six.string_types))
+        self.assertTrue(isinstance(log, str))
 
     def test_list_availability_zone_names(self):
         self.assertEqual(
@@ -232,11 +230,12 @@ class TestCompute(base.BaseFunctionalTest):
             wait=True)
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
-        self.assertEqual(self.flavor.id, server['flavor']['id'])
+        self.assertEqual(self.flavor.name, server['flavor']['original_name'])
         self.assertEqual(server['adminPass'], 'sheiqu9loegahSh')
         self.assertTrue(
             self.user_cloud.delete_server(self.server_name, wait=True))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_get_image_id(self):
         self.assertEqual(
@@ -286,7 +285,8 @@ class TestCompute(base.BaseFunctionalTest):
         self.assertTrue(self.user_cloud.delete_server(server.id, wait=True))
         self._wait_for_detach(volume.id)
         self.assertTrue(self.user_cloud.delete_volume(volume.id, wait=True))
-        self.assertIsNone(self.user_cloud.get_server(server.id))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
         self.assertIsNone(self.user_cloud.get_volume(volume.id))
 
     def _wait_for_detach(self, volume_id):
@@ -324,7 +324,8 @@ class TestCompute(base.BaseFunctionalTest):
         # that is in the process of being deleted.
         if volume:
             self.assertEqual('deleting', volume.status)
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_create_boot_from_volume_preexisting(self):
         self.skipTest('Volume functional tests temporarily disabled')
@@ -351,7 +352,8 @@ class TestCompute(base.BaseFunctionalTest):
         self.assertEqual([], volume['attachments'])
         self._wait_for_detach(volume.id)
         self.assertTrue(self.user_cloud.delete_volume(volume_id))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
         self.assertIsNone(self.user_cloud.get_volume(volume_id))
 
     def test_create_boot_attach_volume(self):
@@ -379,7 +381,8 @@ class TestCompute(base.BaseFunctionalTest):
         self.assertEqual([], volume['attachments'])
         self._wait_for_detach(volume.id)
         self.assertTrue(self.user_cloud.delete_volume(volume_id))
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
         self.assertIsNone(self.user_cloud.get_volume(volume_id))
 
     def test_create_boot_from_volume_preexisting_terminate(self):
@@ -405,7 +408,8 @@ class TestCompute(base.BaseFunctionalTest):
         # that is in the process of being deleted.
         if volume:
             self.assertEqual('deleting', volume.status)
-        self.assertIsNone(self.user_cloud.get_server(self.server_name))
+        srv = self.user_cloud.get_server(self.server_name)
+        self.assertTrue(srv is None or srv.status.lower() == 'deleted')
 
     def test_create_image_snapshot_wait_active(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -481,6 +485,6 @@ class TestCompute(base.BaseFunctionalTest):
         self.add_info_on_exception('usage', usage)
         self.assertIsNotNone(usage)
         self.assertIn('total_hours', usage)
-        self.assertIn('started_at', usage)
-        self.assertEqual(start.isoformat(), usage['started_at'])
+        self.assertIn('start', usage)
+        self.assertEqual(start.isoformat(), usage['start'])
         self.assertIn('location', usage)
