@@ -16,6 +16,8 @@ from openstack.block_storage.v3 import _proxy
 from openstack.block_storage.v3 import backup
 from openstack.block_storage.v3 import capabilities
 from openstack.block_storage.v3 import extension
+from openstack.block_storage.v3 import group
+from openstack.block_storage.v3 import group_snapshot
 from openstack.block_storage.v3 import group_type
 from openstack.block_storage.v3 import limits
 from openstack.block_storage.v3 import quota_set
@@ -39,7 +41,9 @@ class TestVolume(TestVolumeProxy):
         self.verify_get(self.proxy.get_volume, volume.Volume)
 
     def test_volume_find(self):
-        self.verify_find(self.proxy.find_volume, volume.Volume)
+        self.verify_find(self.proxy.find_volume, volume.Volume,
+                         expected_kwargs=dict(
+                             list_base_path='/volumes/detail'))
 
     def test_volumes_detailed(self):
         self.verify_list(
@@ -143,6 +147,92 @@ class TestResourceFilter(TestVolumeProxy):
         )
 
 
+class TestGroup(TestVolumeProxy):
+    def test_group_get(self):
+        self.verify_get(self.proxy.get_group, group.Group)
+
+    def test_group_find(self):
+        self.verify_find(self.proxy.find_group, group.Group)
+
+    def test_groups(self):
+        self.verify_list(self.proxy.groups, group.Group)
+
+    def test_group_create(self):
+        self.verify_create(self.proxy.create_group, group.Group)
+
+    def test_group_create_from_source(self):
+        self._verify(
+            "openstack.block_storage.v3.group.Group.create_from_source",
+            self.proxy.create_group_from_source,
+            method_args=[],
+            expected_args=[self.proxy],
+        )
+
+    def test_group_delete(self):
+        self._verify(
+            "openstack.block_storage.v3.group.Group.delete",
+            self.proxy.delete_group,
+            method_args=['delete_volumes'],
+            expected_args=[self.proxy],
+            expected_kwargs={'delete_volumes': False},
+        )
+
+    def test_group_update(self):
+        self.verify_update(self.proxy.update_group, group.Group)
+
+    def reset_group_state(self):
+        self._verify(self.proxy.reset_group_state, group.Group)
+
+
+class TestGroupSnapshot(TestVolumeProxy):
+    def test_group_snapshot_get(self):
+        self.verify_get(
+            self.proxy.get_group_snapshot, group_snapshot.GroupSnapshot
+        )
+
+    def test_group_snapshot_find(self):
+        self.verify_find(
+            self.proxy.find_group_snapshot, group_snapshot.GroupSnapshot
+        )
+
+    def test_group_snapshots(self):
+        self.verify_list(
+            self.proxy.group_snapshots,
+            group_snapshot.GroupSnapshot,
+            expected_kwargs={},
+        )
+
+    def test_group_snapshots__detailed(self):
+        self.verify_list(
+            self.proxy.group_snapshots,
+            group_snapshot.GroupSnapshot,
+            method_kwargs={'details': True, 'query': 1},
+            expected_kwargs={
+                'query': 1,
+                'base_path': '/group_snapshots/detail',
+            },
+        )
+
+    def test_group_snapshot_create(self):
+        self.verify_create(
+            self.proxy.create_group_snapshot, group_snapshot.GroupSnapshot
+        )
+
+    def test_group_snapshot_delete(self):
+        self.verify_delete(
+            self.proxy.delete_group_snapshot,
+            group_snapshot.GroupSnapshot,
+            False,
+        )
+
+    def test_group_snapshot_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_group_snapshot,
+            group_snapshot.GroupSnapshot,
+            True,
+        )
+
+
 class TestGroupType(TestVolumeProxy):
     def test_group_type_get(self):
         self.verify_get(self.proxy.get_group_type, group_type.GroupType)
