@@ -10,8 +10,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import datetime
 from unittest import mock
+import uuid
+import warnings
 
+from openstack.block_storage.v3 import volume
 from openstack.compute.v2 import _proxy
 from openstack.compute.v2 import aggregate
 from openstack.compute.v2 import availability_zone as az
@@ -30,6 +34,8 @@ from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import server_migration
 from openstack.compute.v2 import server_remote_console
 from openstack.compute.v2 import service
+from openstack.compute.v2 import usage
+from openstack.compute.v2 import volume_attachment
 from openstack import resource
 from openstack.tests.unit import test_proxy_base
 
@@ -451,6 +457,157 @@ class TestService(TestComputeProxy):
         )
 
 
+class TestVolumeAttachment(TestComputeProxy):
+
+    def test_volume_attachment_create(self):
+        self.verify_create(
+            self.proxy.create_volume_attachment,
+            volume_attachment.VolumeAttachment,
+            method_kwargs={'server': 'server_id', 'volume': 'volume_id'},
+            expected_kwargs={
+                'server_id': 'server_id',
+                'volume_id': 'volume_id',
+            },
+        )
+
+    def test_volume_attachment_create__legacy_parameters(self):
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+
+            self.verify_create(
+                self.proxy.create_volume_attachment,
+                volume_attachment.VolumeAttachment,
+                method_kwargs={'server': 'server_id', 'volumeId': 'volume_id'},
+                expected_kwargs={
+                    'server_id': 'server_id',
+                    'volume_id': 'volume_id',
+                },
+            )
+
+            self.assertEqual(1, len(w))
+            self.assertEqual(w[-1].category, DeprecationWarning)
+            self.assertIn(
+                'This method was called with a volume_id or volumeId argument',
+                str(w[-1]),
+            )
+
+    def test_volume_attachment_create__missing_parameters(self):
+        exc = self.assertRaises(
+            TypeError,
+            self.proxy.create_volume_attachment,
+            'server_id',
+        )
+        self.assertIn(
+            'create_volume_attachment() missing 1 required positional argument: volume',  # noqa: E501
+            str(exc),
+        )
+
+    def test_volume_attachment_update(self):
+        self.verify_update(
+            self.proxy.update_volume_attachment,
+            volume_attachment.VolumeAttachment,
+            method_args=[],
+            method_kwargs={'server': 'server_id', 'volume': 'volume_id'},
+            expected_kwargs={
+                'id': 'volume_id',
+                'server_id': 'server_id',
+                'volume_id': 'volume_id',
+            },
+        )
+
+    def test_volume_attachment_delete(self):
+        # We pass objects to avoid the lookup that's done as part of the
+        # handling of legacy option order. We test that legacy path separately.
+        fake_server = server.Server(id=str(uuid.uuid4()))
+        fake_volume = volume.Volume(id=str(uuid.uuid4()))
+
+        self.verify_delete(
+            self.proxy.delete_volume_attachment,
+            volume_attachment.VolumeAttachment,
+            ignore_missing=False,
+            method_args=[fake_server, fake_volume],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'id': fake_volume.id,
+                'server_id': fake_server.id,
+            },
+        )
+
+    def test_volume_attachment_delete__ignore(self):
+        # We pass objects to avoid the lookup that's done as part of the
+        # handling of legacy option order. We test that legacy path separately.
+        fake_server = server.Server(id=str(uuid.uuid4()))
+        fake_volume = volume.Volume(id=str(uuid.uuid4()))
+
+        self.verify_delete(
+            self.proxy.delete_volume_attachment,
+            volume_attachment.VolumeAttachment,
+            ignore_missing=True,
+            method_args=[fake_server, fake_volume],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'id': fake_volume.id,
+                'server_id': fake_server.id,
+            },
+        )
+
+    def test_volume_attachment_delete__legacy_parameters(self):
+        fake_server = server.Server(id=str(uuid.uuid4()))
+        fake_volume = volume.Volume(id=str(uuid.uuid4()))
+
+        with mock.patch.object(
+            self.proxy,
+            'find_server',
+            return_value=None,
+        ) as mock_find_server:
+            # we are calling the method with volume and server ID arguments as
+            # strings and in the wrong order, which results in a query as we
+            # attempt to match the server ID to an actual server before we
+            # switch the argument order once we realize we can't do this
+            self.verify_delete(
+                self.proxy.delete_volume_attachment,
+                volume_attachment.VolumeAttachment,
+                ignore_missing=False,
+                method_args=[fake_volume.id, fake_server.id],
+                method_kwargs={},
+                expected_args=[],
+                expected_kwargs={
+                    'id': fake_volume.id,
+                    'server_id': fake_server.id,
+                },
+            )
+
+            # note that we attempted to call the server with the volume ID but
+            # this was mocked to return None (as would happen in the real
+            # world)
+            mock_find_server.assert_called_once_with(
+                fake_volume.id,
+                ignore_missing=True,
+            )
+
+    def test_volume_attachment_get(self):
+        self.verify_get(
+            self.proxy.get_volume_attachment,
+            volume_attachment.VolumeAttachment,
+            method_args=[],
+            method_kwargs={'server': 'server_id', 'volume': 'volume_id'},
+            expected_kwargs={
+                'id': 'volume_id',
+                'server_id': 'server_id',
+            },
+        )
+
+    def test_volume_attachments(self):
+        self.verify_list(
+            self.proxy.volume_attachments,
+            volume_attachment.VolumeAttachment,
+            method_kwargs={'server': 'server_id'},
+            expected_kwargs={'server_id': 'server_id'},
+        )
+
+
 class TestHypervisor(TestComputeProxy):
 
     def test_hypervisors_not_detailed(self):
@@ -666,7 +823,11 @@ class TestCompute(TestComputeProxy):
             expected_args=[self.proxy])
 
     def test_server_find(self):
-        self.verify_find(self.proxy.find_server, server.Server)
+        self.verify_find(
+            self.proxy.find_server,
+            server.Server,
+            expected_kwargs={'list_base_path': '/servers/detail'},
+        )
 
     def test_server_get(self):
         self.verify_get(self.proxy.get_server, server.Server)
@@ -919,6 +1080,13 @@ class TestCompute(TestComputeProxy):
             method_args=["value"],
             expected_args=[self.proxy])
 
+    def test_server_trigger_dump(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.trigger_crash_dump",
+            self.proxy.trigger_server_crash_dump,
+            method_args=["value"],
+            expected_args=[self.proxy])
+
     def test_get_server_output(self):
         self._verify(
             "openstack.compute.v2.server.Server.get_console_output",
@@ -1084,6 +1252,14 @@ class TestCompute(TestComputeProxy):
     def test_migrations(self):
         self.verify_list(self.proxy.migrations, migration.Migration)
 
+    def test_migrations_kwargs(self):
+        self.verify_list(
+            self.proxy.migrations,
+            migration.Migration,
+            method_kwargs={'host': 'h1'},
+            expected_kwargs={'host': 'h1'},
+        )
+
     def test_fetch_security_groups(self):
         self._verify(
             'openstack.compute.v2.server.Server.fetch_security_groups',
@@ -1105,6 +1281,49 @@ class TestCompute(TestComputeProxy):
             method_args=["value", {'id': 'id', 'name': 'sg'}],
             expected_args=[self.proxy, 'sg'])
 
+    def test_usages(self):
+        self.verify_list(self.proxy.usages, usage.Usage)
+
+    def test_usages__with_kwargs(self):
+        now = datetime.datetime.utcnow()
+        start = now - datetime.timedelta(weeks=4)
+        end = end = now + datetime.timedelta(days=1)
+        self.verify_list(
+            self.proxy.usages,
+            usage.Usage,
+            method_kwargs={'start': start, 'end': end},
+            expected_kwargs={
+                'start': start.isoformat(),
+                'end': end.isoformat()
+            },
+        )
+
+    def test_get_usage(self):
+        self._verify(
+            "openstack.compute.v2.usage.Usage.fetch",
+            self.proxy.get_usage,
+            method_args=['value'],
+            method_kwargs={},
+            expected_args=[self.proxy],
+            expected_kwargs={},
+        )
+
+    def test_get_usage__with_kwargs(self):
+        now = datetime.datetime.utcnow()
+        start = now - datetime.timedelta(weeks=4)
+        end = end = now + datetime.timedelta(days=1)
+        self._verify(
+            "openstack.compute.v2.usage.Usage.fetch",
+            self.proxy.get_usage,
+            method_args=['value'],
+            method_kwargs={'start': start, 'end': end},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                'start': start.isoformat(),
+                'end': end.isoformat()
+            },
+        )
+
     def test_create_server_remote_console(self):
         self.verify_create(
             self.proxy.create_server_remote_console,
@@ -1252,11 +1471,11 @@ class TestServerAction(TestComputeProxy):
         self._verify(
             'openstack.proxy.Proxy._get',
             self.proxy.get_server_action,
-            method_args=['action_id'],
+            method_args=['request_id'],
             method_kwargs={'server': 'server_id'},
             expected_args=[server_action.ServerAction],
             expected_kwargs={
-                'action_id': 'action_id', 'server_id': 'server_id',
+                'request_id': 'request_id', 'server_id': 'server_id',
             },
         )
 
