@@ -12,6 +12,7 @@
 
 from unittest import mock
 
+from openstack.compute.v2 import flavor
 from openstack.compute.v2 import server
 from openstack.image.v2 import image
 from openstack.tests.unit import base
@@ -64,7 +65,6 @@ EXAMPLE = {
         'original_name': 'm1.tiny.specs',
         'ram': 512,
         'swap': 0,
-        'vcpus': 1
     },
     'hostId': '2091634baaccdc4c5a1d57069c833e402921df696b7f970791b12ec6',
     'host_status': 'UP',
@@ -194,7 +194,7 @@ class TestServer(base.TestCase):
         self.assertEqual(EXAMPLE['created'], sot.created_at)
         self.assertEqual(EXAMPLE['config_drive'], sot.has_config_drive)
         self.assertEqual(EXAMPLE['flavorRef'], sot.flavor_id)
-        self.assertEqual(EXAMPLE['flavor'], sot.flavor)
+        self.assertEqual(flavor.Flavor(**EXAMPLE['flavor']), sot.flavor)
         self.assertEqual(EXAMPLE['hostId'], sot.host_id)
         self.assertEqual(EXAMPLE['host_status'], sot.host_status)
         self.assertEqual(EXAMPLE['id'], sot.id)
@@ -251,6 +251,13 @@ class TestServer(base.TestCase):
         self.assertEqual(EXAMPLE['trusted_image_certificates'],
                          sot.trusted_image_certificates)
 
+    def test_to_dict_flavor(self):
+        # Ensure to_dict properly resolves flavor and uses defaults for not
+        # specified flavor proerties.
+        sot = server.Server(**EXAMPLE)
+        dct = sot.to_dict()
+        self.assertEqual(0, dct['flavor']['vcpus'])
+
     def test__prepare_server(self):
         zone = 1
         data = 2
@@ -315,11 +322,14 @@ class TestServer(base.TestCase):
         # Let the translate pass through, that portion is tested elsewhere
         sot._translate_response = lambda arg: arg
 
-        result = sot.rebuild(self.sess, name='noo', admin_password='seekr3t',
-                             image='http://image/1', access_ipv4="12.34.56.78",
-                             access_ipv6="fe80::100",
-                             metadata={"meta var": "meta val"},
-                             user_data="ZWNobyAiaGVsbG8gd29ybGQi")
+        result = sot.rebuild(
+            self.sess, name='noo', admin_password='seekr3t',
+            image='http://image/1', access_ipv4="12.34.56.78",
+            access_ipv6="fe80::100",
+            metadata={"meta var": "meta val"},
+            user_data="ZWNobyAiaGVsbG8gd29ybGQi",
+            preserve_ephemeral=False
+        )
 
         self.assertIsInstance(result, server.Server)
 
@@ -357,7 +367,6 @@ class TestServer(base.TestCase):
                 "name": "nootoo",
                 "imageRef": "http://image/2",
                 "adminPass": "seekr3two",
-                "preserve_ephemeral": False
             }
         }
         headers = {'Accept': ''}
@@ -649,6 +658,18 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion=None)
 
+    def test_lock_with_options(self):
+        sot = server.Server(**EXAMPLE)
+
+        res = sot.lock(self.sess, locked_reason='Because why not')
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {'lock': {'locked_reason': 'Because why not'}}
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion=None)
+
     def test_unlock(self):
         sot = server.Server(**EXAMPLE)
 
@@ -799,6 +820,18 @@ class TestServer(base.TestCase):
         self.sess.post.assert_called_with(
             url, json=body, headers=headers, microversion=None)
 
+    def test_trigger_crash_dump(self):
+        sot = server.Server(**EXAMPLE)
+
+        res = sot.trigger_crash_dump(self.sess)
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {'trigger_crash_dump': None}
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion=None)
+
     def test_get_console_output(self):
         sot = server.Server(**EXAMPLE)
 
@@ -1064,9 +1097,7 @@ class TestServer(base.TestCase):
         }
 
         response.status_code = 200
-        response.json.return_value = {
-            'topology': topology
-        }
+        response.json.return_value = topology
 
         self.sess.get.return_value = response
 
