@@ -10,9 +10,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack.key_manager.v1 import order
 from openstack.tests.unit import base
 
-from openstack.key_manager.v1 import order
 
 ID_VAL = "123"
 SECRET_ID = "5"
@@ -38,10 +38,9 @@ class TestOrder(base.TestCase):
         self.assertIsNone(sot.resource_key)
         self.assertEqual('orders', sot.resources_key)
         self.assertEqual('/orders', sot.base_path)
-        self.assertEqual('key-manager', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
