@@ -27,8 +27,6 @@ from openstack.network.v2 import firewall_rule
 from openstack.network.v2 import flavor
 from openstack.network.v2 import floating_ip
 from openstack.network.v2 import health_monitor
-from openstack.network.v2 import ikepolicy
-from openstack.network.v2 import ipsec_site_connection
 from openstack.network.v2 import l3_conntrack_helper
 from openstack.network.v2 import listener
 from openstack.network.v2 import load_balancer
@@ -60,6 +58,10 @@ from openstack.network.v2 import service_profile
 from openstack.network.v2 import service_provider
 from openstack.network.v2 import subnet
 from openstack.network.v2 import subnet_pool
+from openstack.network.v2 import vpn_endpoint_group
+from openstack.network.v2 import vpn_ike_policy
+from openstack.network.v2 import vpn_ipsec_policy
+from openstack.network.v2 import vpn_ipsec_site_connection
 from openstack.network.v2 import vpn_service
 from openstack import proxy as proxy_base
 from openstack.tests.unit import test_proxy_base
@@ -313,66 +315,6 @@ class TestNetworkHealthMonitor(TestNetworkProxy):
                            health_monitor.HealthMonitor)
 
 
-class TestNetworkSiteConnection(TestNetworkProxy):
-    def test_ipsec_site_connection_create_attrs(self):
-        self.verify_create(self.proxy.create_vpn_ipsec_site_connection,
-                           ipsec_site_connection.IPSecSiteConnection)
-
-    def test_ipsec_site_connection_delete(self):
-        self.verify_delete(self.proxy.delete_vpn_ipsec_site_connection,
-                           ipsec_site_connection.IPSecSiteConnection, False)
-
-    def test_ipsec_site_connection_delete_ignore(self):
-        self.verify_delete(self.proxy.delete_vpn_ipsec_site_connection,
-                           ipsec_site_connection.IPSecSiteConnection, True)
-
-    def test_ipsec_site_connection_find(self):
-        self.verify_find(self.proxy.find_vpn_ipsec_site_connection,
-                         ipsec_site_connection.IPSecSiteConnection)
-
-    def test_ipsec_site_connection_get(self):
-        self.verify_get(self.proxy.get_vpn_ipsec_site_connection,
-                        ipsec_site_connection.IPSecSiteConnection)
-
-    def test_ipsec_site_connections(self):
-        self.verify_list(self.proxy.vpn_ipsec_site_connections,
-                         ipsec_site_connection.IPSecSiteConnection)
-
-    def test_ipsec_site_connection_update(self):
-        self.verify_update(self.proxy.update_vpn_ipsec_site_connection,
-                           ipsec_site_connection.IPSecSiteConnection)
-
-
-class TestNetworkIkePolicy(TestNetworkProxy):
-    def test_ikepolicy_create_attrs(self):
-        self.verify_create(self.proxy.create_vpn_ikepolicy,
-                           ikepolicy.IkePolicy)
-
-    def test_ikepolicy_delete(self):
-        self.verify_delete(self.proxy.delete_vpn_ikepolicy,
-                           ikepolicy.IkePolicy, False)
-
-    def test_ikepolicy_delete_ignore(self):
-        self.verify_delete(self.proxy.delete_vpn_ikepolicy,
-                           ikepolicy.IkePolicy, True)
-
-    def test_ikepolicy_find(self):
-        self.verify_find(self.proxy.find_vpn_ikepolicy,
-                         ikepolicy.IkePolicy)
-
-    def test_ikepolicy_get(self):
-        self.verify_get(self.proxy.get_vpn_ikepolicy,
-                        ikepolicy.IkePolicy)
-
-    def test_ikepolicies(self):
-        self.verify_list(self.proxy.vpn_ikepolicies,
-                         ikepolicy.IkePolicy)
-
-    def test_ikepolicy_update(self):
-        self.verify_update(self.proxy.update_vpn_ikepolicy,
-                           ikepolicy.IkePolicy)
-
-
 class TestNetworkListener(TestNetworkProxy):
     def test_listener_create_attrs(self):
         self.verify_create(self.proxy.create_listener, listener.Listener)
@@ -1632,32 +1574,180 @@ class TestNetworkSubnet(TestNetworkProxy):
                            subnet_pool.SubnetPool)
 
 
+class TestNetworkVpnEndpointGroup(TestNetworkProxy):
+    def test_vpn_endpoint_group_create_attrs(self):
+        self.verify_create(
+            self.proxy.create_vpn_endpoint_group,
+            vpn_endpoint_group.VpnEndpointGroup)
+
+    def test_vpn_endpoint_group_delete(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_endpoint_group,
+            vpn_endpoint_group.VpnEndpointGroup, False)
+
+    def test_vpn_endpoint_group_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_endpoint_group,
+            vpn_endpoint_group.VpnEndpointGroup, True)
+
+    def test_vpn_endpoint_group_find(self):
+        self.verify_find(
+            self.proxy.find_vpn_endpoint_group,
+            vpn_endpoint_group.VpnEndpointGroup)
+
+    def test_vpn_endpoint_group_get(self):
+        self.verify_get(
+            self.proxy.get_vpn_endpoint_group,
+            vpn_endpoint_group.VpnEndpointGroup)
+
+    def test_vpn_endpoint_groups(self):
+        self.verify_list(
+            self.proxy.vpn_endpoint_groups,
+            vpn_endpoint_group.VpnEndpointGroup)
+
+    def test_vpn_endpoint_group_update(self):
+        self.verify_update(
+            self.proxy.update_vpn_endpoint_group,
+            vpn_endpoint_group.VpnEndpointGroup)
+
+
+class TestNetworkVpnSiteConnection(TestNetworkProxy):
+    def test_ipsec_site_connection_create_attrs(self):
+        self.verify_create(
+            self.proxy.create_vpn_ipsec_site_connection,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection)
+
+    def test_ipsec_site_connection_delete(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_ipsec_site_connection,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection, False)
+
+    def test_ipsec_site_connection_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_ipsec_site_connection,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection, True)
+
+    def test_ipsec_site_connection_find(self):
+        self.verify_find(
+            self.proxy.find_vpn_ipsec_site_connection,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection)
+
+    def test_ipsec_site_connection_get(self):
+        self.verify_get(
+            self.proxy.get_vpn_ipsec_site_connection,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection)
+
+    def test_ipsec_site_connections(self):
+        self.verify_list(
+            self.proxy.vpn_ipsec_site_connections,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection)
+
+    def test_ipsec_site_connection_update(self):
+        self.verify_update(
+            self.proxy.update_vpn_ipsec_site_connection,
+            vpn_ipsec_site_connection.VpnIPSecSiteConnection)
+
+
+class TestNetworkVpnIkePolicy(TestNetworkProxy):
+    def test_ike_policy_create_attrs(self):
+        self.verify_create(
+            self.proxy.create_vpn_ike_policy,
+            vpn_ike_policy.VpnIkePolicy)
+
+    def test_ike_policy_delete(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_ike_policy,
+            vpn_ike_policy.VpnIkePolicy, False)
+
+    def test_ike_policy_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_ike_policy,
+            vpn_ike_policy.VpnIkePolicy, True)
+
+    def test_ike_policy_find(self):
+        self.verify_find(
+            self.proxy.find_vpn_ike_policy,
+            vpn_ike_policy.VpnIkePolicy)
+
+    def test_ike_policy_get(self):
+        self.verify_get(
+            self.proxy.get_vpn_ike_policy,
+            vpn_ike_policy.VpnIkePolicy)
+
+    def test_ike_policies(self):
+        self.verify_list(
+            self.proxy.vpn_ike_policies,
+            vpn_ike_policy.VpnIkePolicy)
+
+    def test_ike_policy_update(self):
+        self.verify_update(
+            self.proxy.update_vpn_ike_policy,
+            vpn_ike_policy.VpnIkePolicy)
+
+
+class TestNetworkVpnIpsecPolicy(TestNetworkProxy):
+    def test_ipsec_policy_create_attrs(self):
+        self.verify_create(
+            self.proxy.create_vpn_ipsec_policy,
+            vpn_ipsec_policy.VpnIpsecPolicy)
+
+    def test_ipsec_policy_delete(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_ipsec_policy,
+            vpn_ipsec_policy.VpnIpsecPolicy, False)
+
+    def test_ipsec_policy_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_vpn_ipsec_policy,
+            vpn_ipsec_policy.VpnIpsecPolicy, True)
+
+    def test_ipsec_policy_find(self):
+        self.verify_find(
+            self.proxy.find_vpn_ipsec_policy,
+            vpn_ipsec_policy.VpnIpsecPolicy)
+
+    def test_ipsec_policy_get(self):
+        self.verify_get(
+            self.proxy.get_vpn_ipsec_policy,
+            vpn_ipsec_policy.VpnIpsecPolicy)
+
+    def test_ipsec_policies(self):
+        self.verify_list(
+            self.proxy.vpn_ipsec_policies,
+            vpn_ipsec_policy.VpnIpsecPolicy)
+
+    def test_ipsec_policy_update(self):
+        self.verify_update(
+            self.proxy.update_vpn_ipsec_policy,
+            vpn_ipsec_policy.VpnIpsecPolicy)
+
+
 class TestNetworkVpnService(TestNetworkProxy):
     def test_vpn_service_create_attrs(self):
         self.verify_create(self.proxy.create_vpn_service,
-                           vpn_service.VPNService)
+                           vpn_service.VpnService)
 
     def test_vpn_service_delete(self):
         self.verify_delete(self.proxy.delete_vpn_service,
-                           vpn_service.VPNService, False)
+                           vpn_service.VpnService, False)
 
     def test_vpn_service_delete_ignore(self):
         self.verify_delete(self.proxy.delete_vpn_service,
-                           vpn_service.VPNService, True)
+                           vpn_service.VpnService, True)
 
     def test_vpn_service_find(self):
         self.verify_find(self.proxy.find_vpn_service,
-                         vpn_service.VPNService)
+                         vpn_service.VpnService)
 
     def test_vpn_service_get(self):
-        self.verify_get(self.proxy.get_vpn_service, vpn_service.VPNService)
+        self.verify_get(self.proxy.get_vpn_service, vpn_service.VpnService)
 
     def test_vpn_services(self):
-        self.verify_list(self.proxy.vpn_services, vpn_service.VPNService)
+        self.verify_list(self.proxy.vpn_services, vpn_service.VpnService)
 
     def test_vpn_service_update(self):
         self.verify_update(self.proxy.update_vpn_service,
-                           vpn_service.VPNService)
+                           vpn_service.VpnService)
 
 
 class TestNetworkServiceProvider(TestNetworkProxy):
