@@ -108,11 +108,15 @@ class TestProxyBase(base.TestCase):
         expected_args=None, expected_kwargs=None,
         mock_method="openstack.proxy.Proxy._delete",
     ):
-        method_args = method_args or ['resource_id']
-        method_kwargs = method_kwargs or {}
+        if method_args is None:
+            method_args = ['resource_id']
+        if method_kwargs is None:
+            method_kwargs = {}
         method_kwargs["ignore_missing"] = ignore_missing
-        expected_args = expected_args or method_args.copy()
-        expected_kwargs = expected_kwargs or method_kwargs.copy()
+        if expected_args is None:
+            expected_args = method_args.copy()
+        if expected_kwargs is None:
+            expected_kwargs = method_kwargs.copy()
 
         self._verify(
             mock_method,
@@ -154,7 +158,8 @@ class TestProxyBase(base.TestCase):
             res.fetch.assert_called_once_with(
                 proxy, requires_id=True,
                 base_path=None,
-                error_message=mock.ANY)
+                error_message=mock.ANY,
+                skip_cache=False)
 
     def verify_head(
         self, test_method, resource_type, base_path=None, *,
